#!/usr/bin/python
#
# This file is part of Z-Way project
#
# Created by Poltorak Serguei
#
# Copyright (C) 2010 Z-Wave.Me
# All rights reserved
# info@z-wave.me
#
# This source file is subject to the terms and conditions of the
# Z-Wave.Me Software License Agreement which restricts the manner
# in which it may be used.

import xml.dom.minidom
import csv
import os
import re


def GetSubTagChilds(tagList, tagName):
	return list(filter(lambda x: x.nodeName == tagName, tagList))[0].childNodes

def GetSubTagValue(tagList, tagName):
        return GetSubTagChilds(tagList, tagName)[0].nodeValue
                
def GetSubTagAttrValue(tagList, tagName):
	return GetTagAttribute(list(filter(lambda x: x.nodeName == tagName, tagList)), 'value')

def GetSubTagAttribute(tagList, tagName, attrName):
	return GetTagAttribute(list(filter(lambda x: x.nodeName == tagName, tagList)), attrName)

def GetTagAttribute(tag, attrName):
	return tag[0].getAttribute(attrName)

def CreateXMLIndex():
	"""
	Writes tab separated and XML files with index of all XMLs in the current directory.
	"""

	zddxIndexCSV = "ZDDX.indx"
	zddxIndexXML = "ZDDX.indxml"
	
	##TODO: check XML using XSD: zddxsd = os.path.abspath("%s%s/z-wave_v2.xsd" % (os.getcwd(), device.ZWaveAPI.devices.ZDDXManagedFolder))
	zddxs = list(filter(lambda x: re.match("^.*\.xml$", x), os.listdir(".")))

	try:
		index = csv.writer(open(zddxIndexCSV, 'w'), delimiter='\t')
	except Exception as err:
		print("Error creating index file %s: %s" % (zddxIndex, err))
		return -1
	
	doc = xml.dom.minidom.Document()
	root = doc.createElement("ZDDXIndex")
	doc.appendChild(root)
	
	for zddxf in zddxs:
		try:
			zddx = xml.dom.minidom.parse(zddxf)
			
			rootChilds = zddx.childNodes[0].childNodes

			try:
				deviceDescription = GetSubTagChilds(rootChilds, 'deviceDescription')
			except:
				deviceDescription = None
			
			try:
				brandName = GetSubTagValue(deviceDescription, 'brandName')
			except:
				brandName = "";

			try:
				productName = GetSubTagValue(deviceDescription, 'productName')
			except:
				productName = "";
				
			try:
				deviceImage = GetSubTagAttribute(GetSubTagChilds(rootChilds, 'resourceLinks'), 'deviceImage', 'url')
			except:
				deviceImage = "";

			try:
				manualUrl = GetSubTagAttribute(GetSubTagChilds(rootChilds, 'resourceLinks'), 'manualUrl', 'url')
			except:
				manualUrl = "";

			try:
				productCode = GetSubTagValue(deviceDescription, 'productCode')
			except:
				productCode = "";
			
			
			
			deviceData = GetSubTagChilds(rootChilds, 'deviceData')
			
			try:
				rfFrequency = GetSubTagValue(deviceData, 'rfFrequency')
			except:
				rfFrequency = "";

			# Z-Way relies on this order
			fieldNames = [
				'manufacturerId',
				'productType',
				'productId',
				'basicClass',
				'genericClass',
				'specificClass',
				'appVersion',
				'appSubVersion',
				'protoVersion',
				'protoSubVersion',
				'brandName',
				'productName',
				'deviceImageURL',
				'productCode',
				'rfFrequency',
				'manualUrl',
				'filePath'
			]
			fields = [
				int(GetSubTagAttrValue(deviceData, 'manufacturerId'), 16),
				int(GetSubTagAttrValue(deviceData, 'productType'), 16),
				int(GetSubTagAttrValue(deviceData, 'productId'), 16),
				int(GetSubTagAttrValue(deviceData, 'basicClass'), 16),
				int(GetSubTagAttrValue(deviceData, 'genericClass'), 16),
				int(GetSubTagAttrValue(deviceData, 'specificClass'), 16),
				int(GetSubTagAttrValue(deviceData, 'appVersion'), 16),
				int(GetSubTagAttrValue(deviceData, 'appSubVersion'), 16),
				int(GetSubTagAttrValue(deviceData, 'protoVersion'), 16),
				int(GetSubTagAttrValue(deviceData, 'protoSubVersion'), 16),
				brandName,
				productName,
				deviceImage,
				productCode,
				rfFrequency,
				manualUrl,
				zddxf,
			]
			
			# Write CSV
			index.writerow(fields)

			# Add to XML
			node = doc.createElement('DeviceDescription')
			for i in range(len(fieldNames)):
				node.setAttribute(fieldNames[i], "%s" % fields[i])
			root.appendChild(node)

			zddx.unlink() # it can not raise exceptions

		except Exception as err:
			print("Error loading Z-Wave Device Description XML %s: %s" % (zddxf, err))
			continue

	xmlf = open(zddxIndexXML, "w")
	xmlf.write(doc.toprettyxml())
	xmlf.close()

print("Creating index file for ZDDX")
if CreateXMLIndex() == None:
	print("Done")
else:
	print("Failed")
