//
//  ZWorkerThread.h
//  Part of Z-Bee.C library
//
//  Created by Kirill Rogozhkin on 28/9/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zbee_ezsp_frames_public_h
#define zbee_ezsp_frames_public_h

#include "ZPlatform.h"
#include "ZEzspTypes.h"

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: desired_version
// Desired EZSP Version
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_version(ZBee zbee, ZWBYTE desired_version, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg); 

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: value_id
// Config ID
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_value(ZBee zbee, ZWBYTE value_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get NCP configuration
//
// @param: zbee
// ZBee object instance
//
// @param: id
// Config ID
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_configuration_value(ZBee zbee, ZWBYTE id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Radio Channel
//
// @param: zbee
// ZBee object instance
//
// @param: channel_id
// Channel id from 11 to 26
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_set_radio_channel(ZBee zbee, ZWBYTE channel_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Radio Power
//
// @param: zbee
// ZBee object instance
//
// @param: power
// Power level in dBm
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_set_radio_power(ZBee zbee, ZWBYTE power, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set NCP configuration
//
// @param: zbee
// ZBee object instance
//
// @param: config_id
// Config ID
//
// @param: value
// Config value
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_set_configuration_value(ZBee zbee, ZWBYTE config_id, ZWWORD value, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get manufacturer token
//
// @param: zbee
// ZBee object instance
//
// @param: token_id
// Token ID to request
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_mfg_token(ZBee zbee, ZWBYTE token_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_network_state(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: options
// @default: 0
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_network_init(ZBee zbee, ZWWORD options, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @JSName: NetworkInitNoOption
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_network_init_no_option(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Form new network
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_form_network(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Leafe from existing network
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_leave_network(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_set_initial_security_state(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_network_parameters(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get my node ID
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_node_id(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get my MAC (EUI64)
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_eui64(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: key_type
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_key(ZBee zbee, ZWBYTE key_type, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_get_current_security_state(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: policy_id
// TODO
//
// @param: decision_id
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_set_policy(ZBee zbee, ZWBYTE policy_id, ZWWORD decision_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: remoteEui64
// EUI64 of the node
//
// @param: extendedTimeout
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_set_extended_timeout(ZBee zbee, const ZBEUI64 remoteEui64, ZWBOOL extendedTimeout, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Send unicast packet
//
// @param: zbee
// ZBee object instance
//
// @param: length
// Data length
//
// @param: data
// Data
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_send_unicast(ZBee zbee, size_t length, const ZWBYTE * data, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Open network for new nodes
//
// @param: zbee
// ZBee object instance
//
// @param: duration
// Close the network after this period in seconds
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_permit_joining(ZBee zbee, ZWBYTE duration, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: scanType
// TODO
//
// @param: channelMask
// TODO
//
// @param: duration
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_start_scan(ZBee zbee, ZWBYTE scanType, ZWDWORD channelMask, ZWBYTE duration, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_read_counters(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Send broadcast packet
//
// @param: zbee
// ZBee object instance
//
// @param: length
// Data length
//
// @param: data
// Data
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_send_broadcast(ZBee zbee, size_t length, const ZWBYTE * data, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_clear_key_table(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: ieee
// TODO
//
// @param: key
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_add_transient_link_key(ZBee zbee, const ZBEUI64 ieee, const ZBKey16 key, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// TODO
//
// @param: zbee
// ZBee object instance
//
// @param: ieee
// TODO
//
// @param: key
// TODO
//
// @param: type
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
// @report: TODO
//
ZWEXPORT ZWError zbee_ef_import_transient_key(ZBee zbee, const ZBEUI64 ieee, const ZBKey16 key, ZWBYTE type, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Binding
//
// @param: zbee
// ZBee object instance
//
// @param: index
// The index of a binding table entry.
//
// @param: type
// The type of binding.
//
// @param: local
// The endpoint on the local node.
//
// @param: cluster_id
// A cluster ID that matches one from the local endpoint's simple descriptor. 
//
// @param: remote_endpoint_id
// The endpoint on the remote node (specified by identifier).
//
// @param: destination_ieee
// A 64-bit identifier. This is either the destination EUI64 (for unicasts) or the 64-bit group address (for multicasts).
//
// @param: network_index
// The index of the network the binding belongs to.
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_set_binding(ZBee zbee, ZWBYTE index, ZWBYTE type, ZWBYTE local, ZBClusterId cluster_id, ZWBYTE remote_endpoint_id, const ZBEUI64 destination_ieee, ZWBYTE network_index, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// ZLL Set Radio Idle Mode
//
// @param: zbee
// ZBee object instance
//
// @param: radio_power_mode
// The power mode to be set.
//  0x00 - The radio receiver is switched on.
//  0x01 - The radio receiver is switched off.
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_zll_set_radio_idle_mode(ZBee zbee, ZWBYTE radio_power_mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Value
//
// @param: zbee
// ZBee object instance
//
// @param: value_id
// Value Id
//
// @param: value_length
// Value Length
//
// @param: value_data
// Value Data
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_set_value(ZBee zbee, ZWBYTE value_id, ZWBYTE value_length, const ZWBYTE *value_data, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Concentrator
//
// @param: zbee
// ZBee object instance
//
// @param: on
// Is Enable
//
// @param: concentrator_type
// Concentrator Type:
//  0xFFF8 - A concentrator with insufficient memory to store source routes for the entire network. Route records are sent to the concentrator prior to every inbound APS unicast.
//  0xFFF9 - A concentrator with sufficient memory to store source routes for the entire network. Remote nodes stop sending route records once the concentrator has successfully received one.
//
// @param: min_time
// Min Time between MTORR
//
// @param: max_time
// Max Time between MTORR
//
// @param: route_error_threshold
// The number of route errors that will trigger a re-broadcast of the MTORR
//
// @param: delivery_failure_threshold
// The number of APS delivery failures that will trigger a re-broadcast of the MTORR.
//
// @param: max_hops
// The maximum number of hops that the MTORR broadcast will be allowed to have. A value of 0 will be converted to the EMBER_MAX_HOPS value set by the stack
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_set_concentrator(ZBee zbee, ZWBOOL on, ZWWORD concentrator_type, ZWWORD min_time, ZWWORD max_time, ZWBYTE route_error_threshold, ZWBYTE delivery_failure_threshold, ZWBYTE max_hops, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Clear Transient Link Key
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_clear_transient_link_keys(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Source Route Discovery Mode
//
// @param: zbee
// ZBee object instance
//
// @param: mode
// Source route discovery mode: 
//  0 - off
//  1 - on 
//  2 - reschedule
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_set_source_route_discovery_mode(ZBee zbee, ZWBYTE mode, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Find Key Table Entry
//
// @param: zbee
// ZBee object instance
//
// @param: address
// TODO
//
// @param: link_key
// TODO
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_find_key_table_entry(ZBee zbee, const ZBEUI64 address, ZWBOOL link_key, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Get Multicast Table Entry
//
// @param: zbee
// ZBee object instance
//
// @param: index
// Entry index 
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_get_multicast_table_entry(ZBee zbee, ZWBYTE index, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Set Multicast Table Entry
//
// @param: zbee
// ZBee object instance
//
// @param: index
// Entry index 
//
// @param: multicast_id
// Multicast Address
//
// @param: endpoint
// Endpoint
//
// @param: network_id
// Multicast Address
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_set_multicast_table_entry(ZBee zbee, ZWBYTE index, ZWWORD multicast_id, ZWBYTE endpoint, ZWBYTE network_id, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Custom Frame
//
// @param: zbee
// ZBee object instance
//
// @param: size
// Payload Length
//
// @param: payload
// Payload
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_custom_frame(ZBee zbee, ZWBYTE size, const ZWBYTE *payload, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Custom Frame Load Firmware
//
// @JSName: CustomFrameLoadFirmware
//
// @param: zbee
// ZBee object instance
//
// @param: length
// Payload Length
//
// @param: data
// Firmware
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_custom_frame_load_firmware(ZBee zbee, size_t length, const ZWBYTE *data, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Custom Frame Reset
//
// @JSName: CustomFrameReset
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_custom_frame_reset(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Custom Frame Read Version
//
// @JSName: CustomFrameReadVersion
//
// @param: zbee
// ZBee object instance
//
// @param: successCallback
// @default: NULL
// Custom function to be called on function success
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Custom function to be called on function failure
// NULL if callback is not needed
//
// @param: callbackArg
// Custom argument to be passed to custom function to be called on function success or failure
//
ZWEXPORT ZWError zbee_ef_custom_frame_read_version(ZBee zbee, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Incoming Message Handler Inject
//
// @JSName: IncomingMessageHandlerInject
//
// @param: zbee
// ZBee object instance
//
// @param: seq
// Sequence number
//
// @param: type
// @default: 0
// Message Type: 
// INCOMING_UNICAST - 0x00;
// INCOMING_UNICAST_REPLY - 0x01;
// INCOMING_MULTICAST - 0x02;
// INCOMING_MULTICAST_LOOPBACK - 0x03;
// INCOMING_BROADCAST - 0x04;
// INCOMING_BROADCAST_LOOPBACK - 0x05;
// INCOMING_MANY_TO_ONE_ROUTE_REQUEST - 0x06
//
// @param: profile_id
// @default: 0x0104
// Profile Id
//
// @param: cluster_id
// Cluster Id
//
// @param: src_endpoint_id
// Source Endpoint Id
//
// @param: dst_endpoint_id
// Destination Endpoint Id
//
// @param: options
// @default: 0x0000
// Options
//
// @param: group_id
// @default: 0x0000
// Group Id
//
// @param: aps_seq
// @default: 0x00
// APS Sequence Number
//
// @param: lqi
// @default: 0xFF
// LQI
//
// @param: rssi
// @default: 0xFF
// RSSI
//
// @param: sender
// Sender
//
// @param: binding_index
// @default: 0xFF
// Binding Table Index
//
// @param: addr_index
// @default: 0xFF
// Address Index
//
// @param: size
// Size of Input Data
//
// @param: data
// Input Data
//
// @param: successCallback
// @default: NULL
// Unused
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Unused
// NULL if callback is not needed
//
// @param: callbackArg
// Unused
//
ZWEXPORT ZWError zbee_ef_incoming_message_handler_inject(ZBee zbee, ZWBYTE seq, ZWBYTE type, ZWWORD profile_id, ZWWORD cluster_id, ZWBYTE src_endpoint_id, ZWBYTE dst_endpoint_id, ZWWORD options, ZWWORD group_id, ZWBYTE aps_seq, ZWBYTE lqi, ZWBYTE rssi, ZWWORD sender, ZWBYTE binding_index, ZWBYTE addr_index, ZWBYTE size, const ZWBYTE *data, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

// Incoming Message Handler Inject Raw
//
// @JSName: IncomingMessageHandlerInjectRaw
//
// @param: zbee
// ZBee object instance
//
// @param: size
// Data Size
//
// @param: data
// Data
//
// @param: successCallback
// @default: NULL
// Unused
// NULL if callback is not needed
//
// @param: failureCallback
// @default: NULL
// Unused
// NULL if callback is not needed
//
// @param: callbackArg
// Unused
//
ZWEXPORT ZWError zbee_ef_incoming_message_handler_inject_raw(ZBee zbee, ZWBYTE size, const ZWBYTE *data, ZBJobCustomCallback successCallback, ZBJobCustomCallback failureCallback, void* callbackArg);

#endif