//
//  ZDefsPublic.h
//  Part of Z-Bee.C library
//
//  Created by Kirill Rogozhkin on 20/9/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2012 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zbee_defs_public_h
#define zbee_defs_public_h

#include "ZPlatform.h"
#include "ZData.h"
#include "ZMalloc.h"
#include "ZEzspTypes.h"

#ifdef __cplusplus
extern "C" {
#endif

// Z-Bee defs //

// State of controller
enum _ZBControllerState
{
    ZBIdle = 0,             // Normal operation
    
    ZBAddReady = 1,         // Ready to include device
    ZBAddNodeFound = 2,     // Node found
    ZBAddLearning = 3,      // Assigning NodeId
    ZBAddDone = 4,          // Inclusion done
    
    ZBRemoveReady = 5,      // Ready to exclude device
    ZBRemoveNodeFound = 6,  // Node found
    ZBRemoveLearning = 7,   // Assigning NodeId to zero

    ZBResetting = 20,       // Resetting controller
};

// Use this type to save controller state
typedef int ZBState;

// Types of Device change events
enum _ZBDeviceChangeType
{
    ZBDeviceAdded = 0x01,
    ZBDeviceRemoved = 0x02,
    ZBEndPointAdded = 0x04,
    ZBEndPointRemoved = 0x08,
    ZBClusterAdded = 0x10,
    ZBClusterRemoved = 0x20,
    ZBDDXSaved = 0x100, // this callback notifies on ZDDX data change (to allow main program to purge buffers to disk/flash). For this event deviceId = 0
    ZBEnumerateExisting = 0x200, // this flag makes callback immediately fire for all existing devices classes as if they change profile (works with DeviceProfileChanged only)
};

// Device IEEE
typedef uint8_t ZBEUI64[8];

typedef uint8_t ZBKey16[16];

// ZigBee Network Layer Address
typedef uint16_t ZBNWK;

// ZTimer defs
struct _ZTimer;
typedef struct _ZTimer *ZTimer;

// Cluster Id
typedef uint16_t ZBClusterId;

// Command Id 
typedef ZWBYTE ZBCommandId;

// Manufacturer Code type
typedef uint16_t ZBManufacturerCode;

// List of NULL terminated registered devices Node Id
typedef ZBNWK * ZBDevicesList;

// List of NULL terminated registered instances Id for device
typedef ZWBYTE * ZBEndPointsList;

// List of NULL terminated registered clusters Id for endpoint of device
typedef ZBClusterId * ZBClustersList;

// Z-Bee cluster holder
struct _ZBee;
typedef struct _ZBee *ZBee;

// Definition of termination callback function
typedef void (*ZBTerminationCallback)(const ZBee zbee, void* arg);

// Use this type to store Device change event type
typedef int ZBDeviceChangeType;

// Definition of device change callback function
typedef void (*ZBDeviceCallback)(const ZBee zbee, ZBDeviceChangeType type, ZBNWK node_id, ZWBYTE endpoint_id, ZBClusterId cluster_id, void *arg);

// Definition of custom callback for EZSP Frame and ZCL calls
typedef void (*ZBJobCustomCallback)(const ZBee zbee, ZEzspFrameId id, void* arg);

#define ZB_GLOBAL_ID_READ_ATTRIBUTES                       0x00
#define ZB_GLOBAL_ID_READ_ATTRIBUTES_RESPONSE              0x01
#define ZB_GLOBAL_ID_WRITE_ATTRIBUTES                      0x02
#define ZB_GLOBAL_ID_WRITE_ATTRIBUTES_UNDIVIDED            0x03
#define ZB_GLOBAL_ID_WRITE_ATTRIBUTES_RESPONSE             0x04
#define ZB_GLOBAL_ID_WRITE_ATTRIBUTES_NO_RESPONSE          0x05
#define ZB_GLOBAL_ID_CONFIGURE_REPORTING                   0x06
#define ZB_GLOBAL_ID_CONFIGURE_REPORTING_RESPONSE          0x07
#define ZB_GLOBAL_ID_READ_REPORTING_CONFIGURATION          0x08
#define ZB_GLOBAL_ID_READ_REPORTING_CONFIGURATION_RESPONSE 0x09
#define ZB_GLOBAL_ID_REPORT_ATTRIBUTES                     0x0a
#define ZB_GLOBAL_ID_DEFAULT_RESPONSE                      0x0b
#define ZB_GLOBAL_ID_DISCOVER_ATTRIBUTES                   0x0c
#define ZB_GLOBAL_ID_DISCOVER_ATTRIBUTES_RESPONSE          0x0d
#define ZB_GLOBAL_READ_ATTRIBUTES_STRUCTURED               0x0e
#define ZB_GLOBAL_WRITE_ATTRIBUTES_STRUCTURED              0x0f
#define ZB_GLOBAL_WRITE_ATTRIBUTES_STRUCTURED_RESPONSE     0x10
#define ZB_GLOBAL_DISCOVER_COMMANDS_RECEIVED               0x11
#define ZB_GLOBAL_DISCOVER_COMMANDS_RECEIVED_RESPONSE      0x12
#define ZB_GLOBAL_DISCOVER_COMMANDS_GENERATED              0x13
#define ZB_GLOBAL_DISCOVER_COMMANDS_GENERATED_RESPONSE     0x14
#define ZB_GLOBAL_DISCOVER_ATTRIBUTES_EXTENDED             0x15
#define ZB_GLOBAL_DISCOVER_ATTRIBUTES_EXTENDED_RESPONSE    0x16

// Attribute Id
typedef uint16_t ZBAttributeId;

// AttributeSet Id
typedef uint16_t ZBAttributeSetId;

// ZDO commands Cluster ID

#define ZDO_COMMAND_NETWORK_ADDRESS_REQUEST_CLUSTER_ID 	    0x0000
#define ZDO_COMMAND_IEEE_ADDRESS_REQUEST_CLUSTER_ID 	    0x0001
#define ZDO_COMMAND_NODE_DESCRIPTOR_REQUEST_CLUSTER_ID 	    0x0002
#define ZDO_COMMAND_SIMPLE_DESCRIPTOR_REQUEST_CLUSTER_ID 	0x0004
#define ZDO_COMMAND_ACTIVE_ENDPOINTS_REQUEST_CLUSTER_ID 	0x0005
#define ZDO_COMMAND_MATCH_DESCRIPTOR_REQUEST_CLUSTER_ID 	0x0006
#define ZDO_COMMAND_MQMT_LQI_REQUEST_CLUSTER_ID 	        0x0031
#define ZDO_COMMAND_MQMT_ROUTING_REQUEST_CLUSTER_ID 	    0x0032
#define ZDO_COMMAND_MQMT_LEAVE_REQUEST_CLUSTER_ID 	        0x0034
#define ZDO_COMMAND_MQMT_PERMIT_JOINING_REQUEST_CLUSTER_ID 	0x0036
#define ZDO_COMMAND_MQMT_NETWORK_UPDATE_REQUEST_CLUSTER_ID 	0x0038

#define ZB_ENDPOINT_BROADCAST_ID 0xFF

typedef enum
{
    ZB_BROADCAST_ADDR_ALL_DEVICES                 = 0xFFFF,
    ZB_BROADCAST_ADDR_RESERVED_FFFE               = 0xFFFE,
    ZB_BROADCAST_ADDR_RX_ON_WHEN_IDLE             = 0xFFFD,
    ZB_BROADCAST_ADDR_ALL_ROUTERS_AND_COORDINATOR = 0xFFFC,
    ZB_BROADCAST_ADDR_LOW_POWER_ROUTER            = 0xFFFB,
    ZB_BROADCAST_ADDR_RESERVED_FFFA               = 0xFFFA,
    ZB_BROADCAST_ADDR_RESERVED_FFF9               = 0xFFF9,
    ZB_BROADCAST_ADDR_RESERVED_FFF8               = 0xFFF8,
} ZBBroadcastAddress;

#define NWK_NO_CHANNELS  0x00000000
#define NWK_ALL_CHANNELS 0x07FFF800
#define NWK_CHANNEL_11   0x00000800
#define NWK_CHANNEL_12   0x00001000
#define NWK_CHANNEL_13   0x00002000
#define NWK_CHANNEL_14   0x00004000
#define NWK_CHANNEL_15   0x00008000
#define NWK_CHANNEL_16   0x00010000
#define NWK_CHANNEL_17   0x00020000
#define NWK_CHANNEL_18   0x00040000
#define NWK_CHANNEL_19   0x00080000
#define NWK_CHANNEL_20   0x00100000
#define NWK_CHANNEL_21   0x00200000
#define NWK_CHANNEL_22   0x00400000
#define NWK_CHANNEL_23   0x00800000
#define NWK_CHANNEL_24   0x01000000
#define NWK_CHANNEL_25   0x02000000
#define NWK_CHANNEL_26   0x04000000


// EmberInitialSecurityBitmask //

// This enables ZigBee Standard Security on the node.
#define EMBER_STANDARD_SECURITY_MODE 0x0000
// This enables Distributed Trust Center Mode for the device
// forming the network. (Previously known as EMBER_NO_TRUST_CENTER_MODE)
#define EMBER_DISTRIBUTED_TRUST_CENTER_MODE 0x0002
// This enables a Global Link Key for the Trust Center. All
// nodes will share the same Trust Center Link Key. 
#define EMBER_TRUST_CENTER_GLOBAL_LINK_KEY 0x0004 
// This enables devices that perform MAC Association with a
// pre-configured Network Key to join the network. It is only
// set on the Trust Center.
#define EMBER_PRECONFIGURED_NETWORK_KEY_MODE 0x0008 
// This denotes that the preconfiguredKey is not the actual
// Link Key but a Secret Key known only to the Trust Center.
// It is hashed with the IEEE Address of the destination
// device in order to create the actual Link Key used in
// encryption. This is bit is only used by the Trust Center. The
// joining device need not set this.
#define EMBER_TRUST_CENTER_USES_HASHED_LINK_KEY 0x0084 
// This denotes that the preconfiguredKey element has valid
// data that should be used to configure the initial security
// state.
#define EMBER_HAVE_PRECONFIGURED_KEY 0x0100 
// This denotes that the networkKey element has valid data
// that should be used to configure the initial security state.
#define EMBER_HAVE_NETWORK_KEY 0x0200
// This denotes to a joining node that it should attempt to
// acquire a Trust Center Link Key during joining. This is only
// necessary if the device does not have a pre-configured
// key.
#define EMBER_GET_LINK_KEY_WHEN_JOINING 0x0400
// This denotes that a joining device should only accept an
// encrypted network key from the Trust Center (using its pre-
// configured key). A key sent in-the-clear by the Trust Center
// will be rejected and the join will fail. This option is only valid
// when utilizing a pre-configured key.
#define EMBER_REQUIRE_ENCRYPTED_KEY 0x0800
// This denotes whether the device should NOT reset its
// outgoing frame counters (both NWK and APS) when
// ::emberSetInitialSecurityState() is called. Normally it is
// advised to reset the frame counter before joining a new
// network. However in cases where a device is joining to the
// same network again (but not using
// ::emberRejoinNetwork()) it should keep the NWK and APS
// frame counters stored in its tokens.
#define EMBER_NO_FRAME_COUNTER_RESET 0x1000
// This denotes that the device should obtain its
// preconfigured key from an installation code stored in the
// manufacturing token. The token contains a value that will
// be hashed to obtain the actual preconfigured key. If that
// token is not valid, then the call to
// emberSetInitialSecurityState() will fail.
#define EMBER_GET_PRECONFIGURED_KEY_FROM_INSTALL_CODE 0x2000
// This denotes that the
// ::EmberInitialSecurityState::preconfiguredTrustCenterEui64
// has a value in it containing the trust center EUI64. The
// device will only join a network and accept commands from
// a trust center with that EUI64. Normally this bit is NOT set,
// and the EUI64 of the trust center is learned during the join
// process. When commissioning a device to join onto an
// existing network, which is using a trust center, and without
// sending any messages, this bit must be set and the field
// ::EmberInitialSecurityState::preconfiguredTrustCenterEui64
// must be populated with the appropriate EUI64.
#define EMBER_HAVE_TRUST_CENTER_EUI64 0x0040 


// Attribute Power Source
typedef enum
{
    ZBee_PS_Unknown = 0x00,
    ZBee_PS_MainsSinglePhase = 0x01,
    ZBee_PS_Mains3Phase = 0x02,
    ZBee_PS_Battery = 0x03,
    ZBee_PS_DcSource = 0x04,
    ZBee_PS_EmergencyMainsConstrants = 0x05,
    ZBee_PS_EmergencyMainsAndTransferSwitch = 0x06,
} ZBPowerSource;

// Attribute Physical Enviroment
typedef enum
{
    ZBee_PE_UnspecifiedEnvironment = 0x00,
    ZBee_PE_MirrorZseProfileOrAtrium = 0x01,
    ZBee_PE_Bar = 0x02,
    ZBee_PE_Courtyard = 0x03,
    ZBee_PE_Bathroom = 0x04,
    ZBee_PE_Bedroom = 0x05,
    ZBee_PE_BilliardRoom = 0x06,
    ZBee_PE_UtilityRoom = 0x07,
    ZBee_PE_Cellar = 0x08,
    ZBee_PE_StorageCloset = 0x09,
    ZBee_PE_Theater = 0x0a,
    ZBee_PE_Office1 = 0x0b,
    ZBee_PE_Deck = 0x0c,
    ZBee_PE_Den = 0x0d,
    ZBee_PE_DiningRoom = 0x0e,
    ZBee_PE_ElectricalRoom = 0x0f,
    ZBee_PE_Elevator = 0x10,
    ZBee_PE_Entry = 0x11,
    ZBee_PE_FamilyRoom = 0x12,
    ZBee_PE_MainFloor = 0x13,
    ZBee_PE_Upstairs = 0x14,
    ZBee_PE_Downstairs = 0x15,
    ZBee_PE_BasementOrLowerLevel = 0x16,
    ZBee_PE_Gallery = 0x17,
    ZBee_PE_GameRoom = 0x18,
    ZBee_PE_Garage = 0x19,
    ZBee_PE_Gym = 0x1a,
    ZBee_PE_Hallway = 0x1b,
    ZBee_PE_House = 0x1c,
    ZBee_PE_Kitchen = 0x1d,
    ZBee_PE_LaundryRoom = 0x1e,
    ZBee_PE_Library = 0x1f,
    ZBee_PE_MasterBedroom = 0x20,
    ZBee_PE_MudRoomSmallRoomForCoatsAndBoots = 0x21,
    ZBee_PE_Nursery = 0x22,
    ZBee_PE_Pantry = 0x23,
    ZBee_PE_Office2 = 0x24,
    ZBee_PE_Outside = 0x25,
    ZBee_PE_Pool = 0x26,
    ZBee_PE_Porch = 0x27,
    ZBee_PE_SewingRoom = 0x28,
    ZBee_PE_SittingRoom = 0x29,
    ZBee_PE_Stairway = 0x2a,
    ZBee_PE_Yard = 0x2b,
    ZBee_PE_Attic = 0x2c,
    ZBee_PE_HotTub = 0x2d,
    ZBee_PE_LivingRoom1 = 0x2e,
    ZBee_PE_Sauna = 0x2f,
    ZBee_PE_ShopOrWorkshop = 0x30,
    ZBee_PE_GuestBedroom = 0x31,
    ZBee_PE_GuestBath = 0x32,
    ZBee_PE_PowderRoomHalfBath = 0x33,
    ZBee_PE_BackYard = 0x34,
    ZBee_PE_FrontYard = 0x35,
    ZBee_PE_Patio = 0x36,
    ZBee_PE_Driveway = 0x37,
    ZBee_PE_SunRoom = 0x38,
    ZBee_PE_LivingRoom2 = 0x39,
    ZBee_PE_Spa = 0x3a,
    ZBee_PE_Whirlpool = 0x3b,
    ZBee_PE_Shed = 0x3c,
    ZBee_PE_EquipmentStorage = 0x3d,
    ZBee_PE_HobbyOrCraftRoom = 0x3e,
    ZBee_PE_Fountain = 0x3f,
    ZBee_PE_Pond = 0x40,
    ZBee_PE_ReceptionRoom = 0x41,
    ZBee_PE_BreakfastRoom = 0x42,
    ZBee_PE_Nook = 0x43,
    ZBee_PE_Garden = 0x44,
    ZBee_PE_Balcony = 0x45,
    ZBee_PE_PanicRoom = 0x46,
    ZBee_PE_Terrace = 0x47,
    ZBee_PE_Roof = 0x48,
    ZBee_PE_Toilet = 0x49,
    ZBee_PE_ToiletMain = 0x4a,
    ZBee_PE_OutsideToilet = 0x4b,
    ZBee_PE_ShowerRoom = 0x4c,
    ZBee_PE_Study = 0x4d,
    ZBee_PE_FrontGarden = 0x4e,
    ZBee_PE_BackGarden = 0x4f,
    ZBee_PE_Kettle = 0x50,
    ZBee_PE_Television = 0x51,
    ZBee_PE_Stove = 0x52,
    ZBee_PE_Microwave = 0x53,
    ZBee_PE_Toaster = 0x54,
    ZBee_PE_Vacuum = 0x55,
    ZBee_PE_Appliance = 0x56,
    ZBee_PE_FrontDoor = 0x57,
    ZBee_PE_BackDoor = 0x58,
    ZBee_PE_FridgeDoor = 0x59,
    ZBee_PE_MedicationCabinetDoor = 0x60,
    ZBee_PE_WardrobeDoor = 0x61,
    ZBee_PE_FrontCupboardDoor = 0x62,
    ZBee_PE_OtherDoor = 0x63,
    ZBee_PE_WaitingRoom = 0x64,
    ZBee_PE_TriageRoom = 0x65,
    ZBee_PE_DoctorsOffice = 0x66,
    ZBee_PE_PatientsPrivateRoom = 0x67,
    ZBee_PE_ConsultationRoom = 0x68,
    ZBee_PE_NurseStation = 0x69,
    ZBee_PE_Ward = 0x6a,
    ZBee_PE_Corridor = 0x6b,
    ZBee_PE_OperatingTheatre = 0x6c,
    ZBee_PE_DentalSurgeryRoom = 0x6d,
    ZBee_PE_MedicalImagingRoom = 0x6e,
    ZBee_PE_DecontaminationRoom = 0x6f,
    ZBee_PE_UnknownEnvironment = 0xff,
} ZBPhysicalEnviroment;

// Cluster Identify //
typedef enum
{
    ZBee_CI_Blink = 0x00,
    ZBee_CI_Breathe = 0x01,
    ZBee_CI_Okay = 0x02,
    ZBee_CI_ChannelChange = 0x0b,
    ZBee_CI_FinishEffect = 0xfe,
    ZBee_CI_StopEffect = 0xff,
} ZBIdentifyEffectId;

typedef enum
{
    ZBee_IE_Default = 0x00,
} ZBIdentifyEffectVar;

#ifdef __cplusplus
}
#endif

#endif