//
//  ZEZSPTypes.h
//  Part of Z-Bee.C library
//
//  Created by Kirill Rogozhkin on 26/9/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//  and implements EZSPv8
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zbee_ezsp_types_h
#define zbee_ezsp_types_h

#include "ZPlatform.h"

#define ZEmberEUI64(X) uint8_t X[8]
#define ZExtendedPanId(X) uint8_t X[8]
#define ZEmberKeyData(X)  uint8_t X[16]

typedef uint16_t   ZEzspFrameId;

typedef uint8_t    ZEzspBool;
typedef uint8_t    ZEzspChannel;
typedef uint16_t   ZEmberApsOption;
typedef uint8_t    ZEmberBindingType;
typedef uint8_t    ZEmberCertificate283k1Data;
typedef uint8_t    ZEmberCertificateData;
typedef uint16_t   ZEmberConcentratorType;
typedef uint16_t   ZEmberConfigTxPowerMode;
typedef uint16_t   ZEmberCurrentSecurityBitmask;
typedef uint8_t    ZEmberDeviceUpdate;
typedef uint8_t    ZEmberDutyCycleState;
typedef uint16_t   ZEmberDutyCycleHectoPct;
typedef uint8_t    ZEmberEntropySource;
typedef uint8_t    ZEmberEventUnits;
typedef uint8_t    ZEmberGpKeyType;
typedef uint8_t    ZEmberGpProxyTableEntryStatus;
typedef uint32_t   ZEmberGpSecurityFrameCounter;
typedef uint8_t    ZEmberGpSecurityLevel;
typedef uint8_t    ZEmberGpSinkTableEntryStatus;
typedef uint8_t    ZEmberIncomingMessageType;
typedef uint16_t   ZEmberInitialSecurityBitmask;
typedef uint8_t    ZEmberJoinDecision;
typedef uint8_t    ZEmberJoinMethod;
typedef uint8_t    ZEmberKeyStatus;
typedef uint8_t    ZEmberKeyType;
typedef uint16_t   ZEmberKeyStructBitmask;
typedef uint8_t    ZEmberLibraryId;
typedef uint8_t    ZEmberLibraryStatus;
typedef uint8_t    ZEmberMacPassthroughType;
typedef uint8_t    ZEmberMessageDigest;
typedef uint16_t   ZEmberMulticastId;
typedef uint8_t    ZEmberMultiPhyNwkConfig;
typedef uint16_t   ZEmberNetworkInitBitmask;
typedef uint8_t    ZEmberNetworkStatus;
typedef uint8_t    ZEmberNodeType;
typedef uint8_t    ZEmberOutgoingMessageType;
typedef uint16_t   ZEmberPanId;
typedef uint8_t    ZEmberPrivateKey283k1Data;
typedef uint8_t    ZEmberPrivateKeyData;
typedef uint8_t    ZEmberPublicKey283k1Data;
typedef uint8_t    ZEmberPublicKeyData;
typedef uint8_t    ZEmberRadioPowerMode;
typedef uint8_t    ZEmberSignature283k1Data;
typedef uint8_t    ZEmberSignatureData;
typedef uint8_t    ZEmberSmacData;
typedef uint8_t    ZEmberStatus;
typedef uint8_t    ZEmberZdoConfigurationFlags;
typedef uint8_t    ZEmberZllKeyIndex;
typedef uint16_t   ZEmberZllState;
typedef uint8_t    ZEzspConfigId;
typedef uint16_t   ZEzspDecisionBitMask;
typedef uint8_t    ZEzspDecisionId;
typedef uint16_t   ZEzspEndpointFlags;
typedef uint8_t    ZEzspExtendedValueId;
typedef uint8_t    ZEzspMfgTokenId;
typedef uint8_t    ZEzspNetworkScanType;
typedef uint8_t    ZEzspPolicyId;
typedef uint8_t    ZEzspSourceRouteOverheadInformation;
typedef uint8_t    ZEzspStatus;
typedef uint8_t    ZEzspValueId;
typedef uint8_t    ZEzspZllNetworkOperation;
typedef uint32_t   ZSecureEzspSecurityType;
typedef uint8_t    ZSecureEzspSecurityLevel;

typedef ZEmberNetworkInitBitmask ZEmberNetworkInitStruct;

// Describes the type of ZigBee security key. //

// A shared key between the Trust Center and a device.
#define EMBER_KEY_TYPE_TRUST_CENTER_LINK_KEY 0x01
// A shared secret used for deriving keys between the Trust Center and a
// device
#define EMBER_KEY_TYPE_TRUST_CENTER_MASTER_KEY 0x02
// A shared secret used for deriving keys between the Trust Center and a
// device
#define EMBER_KEY_TYPE_CURRENT_NETWORK_KEY 0x03
// The alternate Network Key that was previously in use, or the newer key
// that will be switched to.
#define EMBER_KEY_TYPE_NEXT_NETWORK_KEY 0x04
// An Application Link Key shared with another (non-Trust Center) device.
#define EMBER_KEY_TYPE_APPLICATION_LINK_KEY 0x05
// An Application Master Key shared secret used to derive an Application
// Link Key.
#define EMBER_KEY_TYPE_APPLICATION_MASTER_KEY 0x06

// Describes the type of ZigBee security key. //

#define EMBER_KEY_STATUS_KEY_STATUS_NONE                               0x00
#define EMBER_KEY_STATUS_APP_LINK_KEY_ESTABLISHED                      0x01
#define EMBER_KEY_STATUS_APP_MASTER_KEY_ESTABLISHED                    0x02  // removed in ver. 5
#define EMBER_KEY_STATUS_TRUST_CENTER_LINK_KEY_ESTABLISHED             0x03
#define EMBER_KEY_STATUS_KEY_ESTABLISHMENT_TIMEOUT                     0x04
#define EMBER_KEY_STATUS_KEY_TABLE_FULL                                0x05
#define EMBER_KEY_STATUS_TC_RESPONDED_TO_KEY_REQUEST                   0x06
#define EMBER_KEY_STATUS_TC_APP_KEY_SENT_TO_REQUESTER                  0x07
#define EMBER_KEY_STATUS_TC_RESPONSE_TO_KEY_REQUEST_FAILED             0x08
#define EMBER_KEY_STATUS_TC_REQUEST_KEY_TYPE_NOT_SUPPORTED             0x09
#define EMBER_KEY_STATUS_TC_NO_LINK_KEY_FOR_REQUESTER                  0x0A
#define EMBER_KEY_STATUS_TC_REQUESTER_EUI64_UNKNOWN                    0x0B
#define EMBER_KEY_STATUS_TC_RECEIVED_FIRST_APP_KEY_REQUEST             0x0C
#define EMBER_KEY_STATUS_TC_TIMEOUT_WAITING_FOR_SECOND_APP_KEY_REQUEST 0x0D
#define EMBER_KEY_STATUS_TC_NON_MATCHING_APP_KEY_REQUEST_RECEIVED      0x0E
#define EMBER_KEY_STATUS_TC_FAILED_TO_SEND_APP_KEYS                    0x0F  
#define EMBER_KEY_STATUS_TC_FAILED_TO_STORE_APP_KEY_REQUEST            0x10
#define EMBER_KEY_STATUS_TC_REJECTED_APP_KEY_REQUEST                   0x11           
#define EMBER_KEY_STATUS_TC_FAILED_TO_GENERATE_NEW_KEY                 0x12
#define EMBER_KEY_STATUS_TC_FAILED_TO_SEND_TC_KEY                      0x13
#define EMBER_KEY_STATUS_TRUST_CENTER_IS_PRE_R21                       0x1E
#define EMBER_KEY_STATUS_TC_REQUESTER_VERIFY_KEY_TIMEOUT               0x32
#define EMBER_KEY_STATUS_TC_REQUESTER_VERIFY_KEY_FAILURE               0x33
#define EMBER_KEY_STATUS_TC_REQUESTER_VERIFY_KEY_SUCCESS               0x34
#define EMBER_KEY_STATUS_VERIFY_LINK_KEY_FAILURE                       0x64
#define EMBER_KEY_STATUS_VERIFY_LINK_KEY_SUCCESS                       0x65

// The status of the device update. //

#define EMBER_DEVICE_UPDATE_STANDARD_SECURITY_SECURED_REJOIN 0x0
#define EMBER_DEVICE_UPDATE_STANDARD_SECURITY_UNSECURED_JOIN 0x1
#define EMBER_DEVICE_UPDATE_DEVICE_LEFT 0x2
#define EMBER_DEVICE_UPDATE_STANDARD_SECURITY_UNSECURED_REJOIN 0x3


// Decision made by the trust center when a node attempts to join. //

// Allow the node to join. The joining node should have a pre-configured key. 
// The security data sent to it will be encrypted with that key.
#define EMBER_USE_PRECONFIGURED_KEY 0x00
// Allow the node to join. Send the network key in-the-clear to the joining device.
#define EMBER_SEND_KEY_IN_THE_CLEAR 0x01
// Deny join.
#define EMBER_DENY_JOIN 0x02
// Take no action.
#define EMBER_NO_ACTION 0x03


// Defines the events reported to the application by the //
// readAndClearCounters command.                         //

// The MAC received a broadcast.
#define EMBER_COUNTER_TYPE_MAC_RX_BROADCAST 0
// The MAC transmitted a broadcast.
#define EMBER_COUNTER_TYPE_MAC_TX_BROADCAST 1
// The MAC received a unicast.
#define EMBER_COUNTER_TYPE_MAC_RX_UNICAST 2
// The MAC successfully transmitted a unicast.
#define EMBER_COUNTER_TYPE_MAC_TX_UNICAST_SUCCESS 3
// The MAC retried a unicast.
#define EMBER_COUNTER_TYPE_MAC_TX_UNICAST_RETRY 4
// The MAC unsuccessfully transmitted a unicast.
#define EMBER_COUNTER_TYPE_MAC_TX_UNICAST_FAILED 5
// The APS layer received a data broadcast.
#define EMBER_COUNTER_TYPE_APS_DATA_RX_BROADCAST 6
// The APS layer transmitted a data broadcast.
#define EMBER_COUNTER_TYPE_APS_DATA_TX_BROADCAST 7
// The APS layer received a data unicast.
#define EMBER_COUNTER_TYPE_APS_DATA_RX_UNICAST 8
// The APS layer successfully transmitted a data unicast.
#define EMBER_COUNTER_TYPE_APS_DATA_TX_UNICAST_SUCCESS 9
// The APS layer retried a data unicast.
#define EMBER_COUNTER_TYPE_APS_DATA_TX_UNICAST_RETRY 10
// The APS layer unsuccessfully transmitted a data unicast.
#define EMBER_COUNTER_TYPE_APS_DATA_TX_UNICAST_FAILED 11
// The network layer successfully submitted a new route discovery to the MAC.
#define EMBER_COUNTER_TYPE_ROUTE_DISCOVERY_INITIATED 12
// An entry was added to the neighbor table.
#define EMBER_COUNTER_TYPE_NEIGHBOR_ADDED 13
// An entry was removed from the neighbor table.
#define EMBER_COUNTER_TYPE_NEIGHBOR_REMOVED 14
// A neighbor table entry became stale because it had not been heard from.
#define EMBER_COUNTER_TYPE_NEIGHBOR_STALE 15
// A node joined or rejoined to the network via this node.
#define EMBER_COUNTER_TYPE_JOIN_INDICATION 16
// An entry was removed from the child table.
#define EMBER_COUNTER_TYPE_CHILD_REMOVED 17
// EZSP-UART only. An overflow error occurred in the UART.
#define EMBER_COUNTER_TYPE_ASH_OVERFLOW_ERROR 18
// EZSP-UART only. A framing error occurred in the UART.
#define EMBER_COUNTER_TYPE_ASH_FRAMING_ERROR 19
// EZSP-UART only. An overrun error occurred in the UART.
#define EMBER_COUNTER_TYPE_ASH_OVERRUN_ERROR 20
// A message was dropped at the network layer because the NWK frame counter
// was not higher than the last message seen from that source.
#define EMBER_COUNTER_TYPE_NWK_FRAME_COUNTER_FAILURE 21
// A message was dropped at the APS layer because the APS frame counter was
// not higher than the last message seen from that source.
#define EMBER_COUNTER_TYPE_APS_FRAME_COUNTER_FAILURE 22
// Utility counter for general debugging use.
#define EMBER_COUNTER_TYPE_UTILITY 23
// A message was dropped at the APS layer because it had APS encryption but
// the key associated with the sender has not been authenticated, and thus
// the key is not authorized for use in APS data messages.
#define EMBER_COUNTER_TYPE_APS_LINK_KEY_NOT_AUTHORIZED 24
// A NWK encrypted message was received but dropped because decryption
// failed.
#define EMBER_COUNTER_TYPE_NWK_DECRYPTION_FAILURE 25
// An APS encrypted message was received but dropped because decryption
// failed.
#define EMBER_COUNTER_TYPE_APS_DECRYPTION_FAILURE 26
// The number of times we failed to allocate a set of linked packet buffers.
// This doesn't necessarily mean that the packet buffer count was 0 at the
// time, but that the number requested was greater than the number free.
#define EMBER_COUNTER_TYPE_ALLOCATE_PACKET_BUFFER_FAILURE 27
// The number of relayed unicast packets.
#define EMBER_COUNTER_TYPE_RELAYED_UNICAST 28
// The number of times we dropped a packet due to reaching
// the preset PHY to MAC queue limit (emMaxPhyToMacQueueLength).
#define EMBER_COUNTER_TYPE_PHY_TO_MAC_QUEUE_LIMIT_REACHED 29
// The number of times we dropped a packet due to the
// packet-validate library checking a packet and rejecting it
// due to length or other formatting problems.
#define EMBER_COUNTER_TYPE_PACKET_VALIDATE_LIBRARY_DROPPED_COUNT 30
// The number of times the NWK retry queue is full and a
// new message failed to be added.
#define EMBER_COUNTER_TYPE_TYPE_NWK_RETRY_OVERFLOW 31
// The number of times the PHY layer was unable to transmitmber of times the PHY layer was unable to transmit
// due to a failed CCA.
#define EMBER_COUNTER_TYPE_PHY_CCA_FAIL_COUNT 32
// The number of times a NWK broadcast was dropped because
// the broadcast table was full.
#define EMBER_COUNTER_TYPE_BROADCAST_TABLE_FULL 33
// The number of low priority packet traffic arbitration requests.
#define EMBER_COUNTER_TYPE_PTA_LO_PRI_REQUESTED 34
// The number of high priority packet traffic arbitration requests.
#define EMBER_COUNTER_TYPE_PTA_HI_PRI_REQUESTED 35
// The number of low priority packet traffic arbitration requests denied.
#define EMBER_COUNTER_TYPE_PTA_LO_PRI_DENIED 36
// The number of high priority packet traffic arbitration requests denied.
#define EMBER_COUNTER_TYPE_PTA_HI_PRI_DENIED 37
// The number of aborted low priority packet traffic arbitration transmissions.
#define EMBER_COUNTER_TYPE_PTA_LO_PRI_TX_ABORTED 38
// The number of aborted high priority packet traffic arbitration transmissions.
#define EMBER_COUNTER_TYPE_PTA_HI_PRI_TX_ABORTED 39
// The number of times an address conflict has caused node_id change, and an address
// conflict error is sent
#define EMBER_COUNTER_TYPE_ADDRESS_CONFLICT_SENT 40

// The type of method used for joining. //

// Normally devices use MAC Association to join a network, which respects
// the "permit joining" flag in the MAC Beacon. For mobile nodes this value
// causes the device to use an Ember Mobile Node Join, which is functionally
// equivalent to a MAC association. This value should be used by default.
#define EMBER_JOIN_METHOD_USE_MAC_ASSOCIATION 0x0
// For those networks where the "permit joining" flag is never turned on,
// they will need to use a ZigBee NWK Rejoin. This value causes the rejoin
// to be sent without NWK security and the Trust Center will be asked to
// send the NWK key to the device. The NWK key sent to the device can be
// encrypted with the device's corresponding Trust Center link key. That is
// determined by the ::EmberJoinDecision on the Trust Center returned by the
// ::emberTrustCenterJoinHandler(). For a mobile node this value will cause
// it to use an Ember Mobile node rejoin, which is functionally equivalent.
#define EMBER_JOIN_METHOD_USE_NWK_REJOIN 0x1
// For those networks where the "permit joining" flag is never turned on,
// they will need to use a NWK Rejoin. If those devices have been
// preconfigured with the NWK key (including sequence number) they can use a
// secured rejoin. This is only necessary for end devices since they need a
// parent. Routers can simply use the ::USE_NWK_COMMISSIONING join method
// below.
#define EMBER_JOIN_METHOD_USE_NWK_REJOIN_HAVE_NWK_KEY 0x2
// For those networks where all network and security information is known
// ahead of time, a router device may be commissioned such that it does not
// need to send any messages to begin communicating on the network.
#define EMBER_JOIN_METHOD_USE_CONFIGURED_NWK_STATE 0x3


// Bitmask options for emberNetworkInit(). //

// No options for Network Init
#define EMBER_NETWORK_BITMASK_INIT_NO_OPTIONS 0x0000
// Save parent info (node ID and EUI64) in a token during joining/rejoin,
// and restore on reboot.
#define EMBER_NETWORK_BITMASK_INIT_PARENT_INFO_IN_TOKEN 0x0001
// Send a rejoin request as an end device on reboot if parent information is
// persisted. ZB3 end devices must rejoin on reboot.
#define EMBER_NETWORK_BITMASK_INIT_END_DEVICE_REJOIN_ON_REBOOT 0x0002


// This is the Current Security Bitmask that details the use of various //
// security features.                                                   //

// This denotes that the device is running in a network with ZigBee Standard
// Security.
#define EMBER_CURRENT_SECURITY_BITMASK_STANDARD_SECURITY_MODE 0x0000
// This denotes that the device is running in a network with ZigBee High
// Security.
#define EMBER_CURRENT_SECURITY_BITMASK_HIGH_SECURITY_MODE 0x0001
// This denotes that the device is running in a network without a
// centralized Trust Center.
#define EMBER_CURRENT_SECURITY_BITMASK_DISTRIBUTED_TRUST_CENTER_MODE 0x0002
// This denotes that the device has a Global Link Key. The Trust Center Link
// Key is the same across multiple nodes.
#define EMBER_CURRENT_SECURITY_BITMASK_GLOBAL_LINK_KEY 0x0004
// This denotes that the node has a Trust Center Link Key.
#define EMBER_CURRENT_SECURITY_BITMASK_HAVE_TRUST_CENTER_LINK_KEY 0x0010
#define EMBER_CURRENT_SECURITY_BITMASK_UNKNOWN_1 0x0020
#define EMBER_CURRENT_SECURITY_BITMASK_UNKNOWN_2 0x0040
// This denotes that the Trust Center is using a Hashed Link Key.
#define EMBER_CURRENT_SECURITY_BITMASK_TRUST_CENTER_USES_HASHED_LINK_KEY 0x0084

// The possible join states for a node.
// The node is not associated with a network in any way.
#define EMBER_NETWORK_STATE_NO_NETWORK 0x00
// The node is currently attempting to join a network.
#define EMBER_NETWORK_STATE_JOINING_NETWORK 0x01
// The node is joined to a network.
#define EMBER_NETWORK_STATE_JOINED_NETWORK 0x02
// The node is an end device joined to a network but its parent is not
// responding.
#define EMBER_NETWORK_STATE_JOINED_NETWORK_NO_PARENT 0x03
// The node is in the process of leaving its current network.
#define EMBER_NETWORK_STATE_LEAVING_NETWORK 0x04


// Network configuration for the desired radio interface for multi phy network. //

// Enable broadcast support on Routers
#define EMBER_MULTI_PHY_NWK_CONFIG_BROADCAST_SUPPORT 0x01

// Duty cycle states. //

// No Duty cycle tracking or metrics are taking place
#define EMBER_DUTY_CYCLE_STATE_TRACKING_OFF 0
// Duty Cycle is tracked and has not exceeded any thresholds.
#define EMBER_DUTY_CYCLE_STATE_LBT_NORMAL 1
// We have exceeded the limited threshold of our total duty cycle allotment.
#define EMBER_DUTY_CYCLE_STATE_LBT_LIMITED_THRESHOLD_REACHED 2
// We have exceeded the critical threshold of our total duty cycle allotment.
#define EMBER_DUTY_CYCLE_STATE_LBT_CRITICAL_THRESHOLD_REACHED 3
// We have reached the suspend limit and are blocking all outbound transmissions.
#define EMBER_DUTY_CYCLE_STATE_LBT_SUSPEND_LIMIT_REACHED 4

// Radio power mode. //

// The radio receiver is switched on.
#define EMBER_RADIO_POWER_MODE_RX_ON 0
// The radio receiver is switched off.
#define EMBER_RADIO_POWER_MODE_OFF 1

// Entropy sources. //

// Entropy source error
#define EMBER_ENTROPY_SOURCE_ERROR 0
// Entropy source is the radio.
#define EMBER_ENTROPY_SOURCE_RADIO 1
// Entropy source is the TRNG powered by mbed TLS.
#define EMBER_ENTROPY_SOURCE_MBEDTLS_TRNG 2
// Entropy source is powered by mbed TLS, the source is not TRNG.
#define EMBER_ENTROPY_SOURCE_MBEDTLS 3

// Identifies a configuration value. //

// The number of packet buffers available to the stack.  When set to the
// special value 0xFF, the NCP will allocate all remaining configuration RAM
// towards packet buffers, such that the resulting count will be the largest
// whole number of packet buffers that can fit into the available memory.
#define EZSP_CONFIG_PACKET_BUFFER_COUNT 0x01
// The maximum number of router neighbors the stack can keep track of. A
// neighbor is a node within radio range.
#define EZSP_CONFIG_NEIGHBOR_TABLE_SIZE 0x02
// The maximum number of APS retried messages the stack can be transmitting
// at any time.
#define EZSP_CONFIG_APS_UNICAST_MESSAGE_COUNT 0x03
// The maximum number of non-volatile bindings supported by the stack.
#define EZSP_CONFIG_BINDING_TABLE_SIZE 0x04
// The maximum number of EUI64 to network address associations that the
// stack can maintain for the application. (Note, the total number of such
// address associations maintained by the NCP is the sum of the value of
// this setting and the value of ::CONFIG_TRUST_CENTER_ADDRESS_CACHE_SIZE).
#define EZSP_CONFIG_ADDRESS_TABLE_SIZE 0x05
// The maximum number of multicast groups that the device may be a member
// of.
#define EZSP_CONFIG_MULTICAST_TABLE_SIZE 0x06
// The maximum number of destinations to which a node can route messages.
// This includes both messages originating at this node and those relayed
// for others.
#define EZSP_CONFIG_ROUTE_TABLE_SIZE 0x07
// The number of simultaneous route discoveries that a node will support.
#define EZSP_CONFIG_DISCOVERY_TABLE_SIZE 0x08
// Specifies the stack profile.
#define EZSP_CONFIG_STACK_PROFILE 0x0C
// The security level used for security at the MAC and network layers. The
// supported values are 0 (no security) and 5 (payload is encrypted and a
// four-byte MIC is used for authentication).
#define EZSP_CONFIG_SECURITY_LEVEL 0x0D
// The maximum number of hops for a message.
#define EZSP_CONFIG_MAX_HOPS 0x10
// The maximum number of end device children that a router will support.
#define EZSP_CONFIG_MAX_END_DEVICE_CHILDREN 0x11
// The maximum amount of time that the MAC will hold a message for indirect
// transmission to a child.
#define EZSP_CONFIG_INDIRECT_TRANSMISSION_TIMEOUT 0x12
// The maximum amount of time that an end device child can wait between
// polls. If no poll is heard within this timeout, then the parent removes
// the end device from its tables.
#define EZSP_CONFIG_END_DEVICE_POLL_TIMEOUT 0x13
// Enables boost power mode and/or the alternate transmitter output.
#define EZSP_CONFIG_TX_POWER_MODE 0x17
// 0: Allow this node to relay messages. 1: Prevent this node from relaying
// messages.
#define EZSP_CONFIG_DISABLE_RELAY 0x18
// The maximum number of EUI64 to network address associations that the
// Trust Center can maintain.  These address cache entries are reserved for
// and reused by the Trust Center when processing device join/rejoin
// authentications. This cache size limits the number of overlapping joins
// the Trust Center can process within a narrow time window (e.g. two
// seconds), and thus should be set to the maximum number of near
// simultaneous joins the Trust Center is expected to accommodate. (Note,
// the total number of such address associations maintained by the NCP is
// the sum of the value of this setting and the value of
// ::CONFIG_ADDRESS_TABLE_SIZE.)
#define EZSP_CONFIG_TRUST_CENTER_ADDRESS_CACHE_SIZE 0x19
// The size of the source route table.
#define EZSP_CONFIG_SOURCE_ROUTE_TABLE_SIZE 0x1A
// The number of blocks of a fragmented message that can be sent in a single
// window.
#define EZSP_CONFIG_FRAGMENT_WINDOW_SIZE 0x1C
// The time the stack will wait (in milliseconds) between sending blocks of
// a fragmented message.
#define EZSP_CONFIG_FRAGMENT_DELAY_MS 0x1D
// The size of the Key Table used for storing individual link keys (if the
// device is a Trust Center) or Application Link Keys (if the device is a
// normal node).
#define EZSP_CONFIG_KEY_TABLE_SIZE 0x1E
// The APS ACK timeout value. The stack waits this amount of time between
// resends of APS retried messages.
#define EZSP_CONFIG_APS_ACK_TIMEOUT 0x1F
// The duration of an active scan, in the units used by the 15.4 scan
// parameter (((1 << duration) + 1) * 15ms). This also controls the jitter
// used when responding to a beacon request.
#define EZSP_CONFIG_ACTIVE_SCAN_DURATION 0x20
// The time the coordinator will wait (in seconds) for a second end device
// bind request to arrive.
#define EZSP_CONFIG_END_DEVICE_BIND_TIMEOUT 0x21
// The number of PAN id conflict reports that must be received by the
// network manager within one minute to trigger a PAN id change.
#define EZSP_CONFIG_PAN_ID_CONFLICT_REPORT_THRESHOLD 0x22
// The timeout value in minutes for how long the Trust Center or a normal
// node waits for the ZigBee Request Key to complete. On the Trust Center
// this controls whether or not the device buffers the request, waiting for
// a matching pair of ZigBee Request Key. If the value is non-zero, the
// Trust Center buffers and waits for that amount of time. If the value is
// zero, the Trust Center does not buffer the request and immediately
// responds to the request.  Zero is the most compliant behavior.
#define EZSP_CONFIG_REQUEST_KEY_TIMEOUT 0x24
// This value indicates the size of the runtime modifiable certificate
// table. Normally certificates are stored in MFG tokens but this table can
// be used to field upgrade devices with new Smart Energy certificates.
// This value cannot be set, it can only be queried.
#define EZSP_CONFIG_CERTIFICATE_TABLE_SIZE 0x29
// This is a bitmask that controls which incoming ZDO request messages are
// passed to the application. The bits are defined in the
// EmberZdoConfigurationFlags enumeration. To see if the application is
// required to send a ZDO response in reply to an incoming message, the
// application must check the APS options bitfield within the
// incomingMessageHandler callback to see if the
// APS_OPTION_ZDO_RESPONSE_REQUIRED flag is set.
#define EZSP_CONFIG_APPLICATION_ZDO_FLAGS 0x2A
// The maximum number of broadcasts during a single broadcast timeout
// period.
#define EZSP_CONFIG_BROADCAST_TABLE_SIZE 0x2B
// The size of the MAC filter list table.
#define EZSP_CONFIG_MAC_FILTER_TABLE_SIZE 0x2C
// The number of supported networks.
#define EZSP_CONFIG_SUPPORTED_NETWORKS 0x2D
// Whether multicasts are sent to the RxOnWhenIdle=true address (0xFFFD) or,
// the sleepy broadcast address (0xFFFF). The RxOnWhenIdle=true address is,
// the ZigBee compliant destination for multicasts.
#define EZSP_CONFIG_SEND_MULTICASTS_TO_SLEEPY_ADDRESS 0x2E
// ZLL group address initial configuration.
#define EZSP_CONFIG_ZLL_GROUP_ADDRESSES 0x2F
// ZLL rssi threshold initial configuration.
#define EZSP_CONFIG_ZLL_RSSI_THRESHOLD 0x30
// Toggles the mtorr flow control in the stack.
// The maximum number of pairings supported by the stack. Controllers
// must support at least one pairing table entry while targets must
// support at least five.
#define EZSP_CONFIG_MTORR_FLOW_CONTROL 0x33
// Setting the retry queue size.
#define EZSP_CONFIG_RETRY_QUEUE_SIZE 0x34
// Setting the new broadcast entry threshold.
#define EZSP_CONFIG_NEW_BROADCAST_ENTRY_THRESHOLD 0x35
// The length of time, in seconds, that a trust center will store a
// transient link key that a device can use to join its network. A transient
// key is added with a call to emberAddTransientLinkKey. After the transient
// key is added, it will be removed once this amount of time has passed. A
// joining device will not be able to use that key to join until it is added
// again on the trust center. The default value is 300 seconds, i.e., 5
// minutes.
#define EZSP_CONFIG_TRANSIENT_KEY_TIMEOUT_S 0x36
// The number of passive acknowledgements to record from neighbors before we stop
// re-transmitting broadcasts
#define EZSP_CONFIG_BROADCAST_MIN_ACKS_NEEDED 0x37
// The length of time, in seconds, that a trust center will allow a Trust Center
// (insecure) rejoin for a device that is using the well-known link key. This timeout
// takes effect once rejoins using the well-known key has been allowed. This command
// updates the emAllowTcRejoinsUsingWellKnownKeyTimeoutSec value.
#define EZSP_CONFIG_TC_REJOINS_USING_WELL_KNOWN_KEY_TIMEOUT_S 0x38
// Valid range of a CTUNE value is 0x0000-0x01FF. Higher order bits (0xFE00) of the
// 16-bit value are ignored.
#define EZSP_CONFIG_CTUNE_VALUE 0x39
// To configure non trust center node to assume a concentrator type of the trust
// center it join to, until it receive many-to-one route request from the trust
// center. For the trust center node, concentrator type is configured from the
// concentrator plugin. The stack by default assumes trust center be a low RAM
// concentrator that make other devices send route record to the trust center even
// without receiving a many-to-one route request. The default concentrator type can
// be changed by setting appropriate EmberAssumeTrustCenterConcentratorType config
// value.
#define EZSP_CONFIG_ASSUME_TC_CONCENTRATOR_TYPE 0x40

// Identifies a value. //

// The contents of the node data stack token.
#define EZSP_VALUE_ID_TOKEN_STACK_NODE_DATA 0x00
// The types of MAC passthrough messages that the host wishes to receive.
#define EZSP_VALUE_ID_MAC_PASSTHROUGH_FLAGS 0x01
// The source address used to filter legacy EmberNet messages when the
// MAC_PASSTHROUGH_EMBERNET_SOURCE flag is set in VALUE_MAC_PASSTHROUGH_FLAGS.
#define EZSP_VALUE_ID_EMBERNET_PASSTHROUGH_SOURCE_ADDRESS 0x02
// The number of available message buffers.
#define EZSP_VALUE_ID_FREE_BUFFERS 0x03
// Selects sending synchronous callbacks in ezsp-uart.
#define EZSP_VALUE_ID_UART_SYNCH_CALLBACKS 0x04
// The maximum incoming transfer size for the local node.
#define EZSP_VALUE_ID_MAXIMUM_INCOMING_TRANSFER_SIZE 0x05
// The maximum outgoing transfer size for the local node.
#define EZSP_VALUE_ID_MAXIMUM_OUTGOING_TRANSFER_SIZE 0x06
// A boolean indicating whether stack tokens are written to persistent
// storage as they change.
#define EZSP_VALUE_ID_STACK_TOKEN_WRITING 0x07
// A read-only value indicating whether the stack is currently performing a rejoin.
#define EZSP_VALUE_ID_STACK_IS_PERFORMING_REJOIN 0x08
// A list of EmberMacFilterMatchData values.
#define EZSP_VALUE_ID_MAC_FILTER_LIST 0x09
// The Ember Extended Security Bitmask.
#define EZSP_VALUE_ID_EXTENDED_SECURITY_BITMASK 0x0A
// The node short ID.
#define EZSP_VALUE_ID_NODE_SHORT_ID 0x0B
// The descriptor capability of the local node.
#define EZSP_VALUE_ID_DESCRIPTOR_CAPABILITY 0x0C
// The stack device request sequence number of the local node.
#define EZSP_VALUE_ID_STACK_DEVICE_REQUEST_SEQUENCE_NUMBER 0x0D
// Enable or disable radio hold-off.
#define EZSP_VALUE_ID_RADIO_HOLD_OFF 0x0E
// The flags field associated with the endpoint data.
#define EZSP_VALUE_ID_ENDPOINT_FLAGS 0x0F
// Enable/disable the Mfg security config key settings.
#define EZSP_VALUE_ID_MFG_SECURITY_CONFIG 0x10
// Retrieves the version information from the stack on the NCP.
#define EZSP_VALUE_ID_VERSION_INFO 0x11
// This will get/set the rejoin reason noted by the host for a subsequent call to
// emberFindAndRejoinNetwork(). After a call to emberFindAndRejoinNetwork() the
// host's rejoin reason will be set to REJOIN_REASON_NONE. The NCP will store the
// rejoin reason used by the call to emberFindAndRejoinNetwork()
#define EZSP_VALUE_ID_NEXT_HOST_REJOIN_REASON 0x12
// This is the reason that the last rejoin took place. This value may only be
// retrieved, not set. The rejoin may have been initiated by the stack (NCP) or the
// application (host). If a host initiated a rejoin the reason will be set by default
// to REJOIN_DUE_TO_APP_EVENT_1. If the application wishes to denote its own rejoin
// reasons it can do so by calling ezspSetValue(VALUE_HOST_REJOIN_REASON,
// REJOIN_DUE_TO_APP_EVENT_X). X is a number corresponding to one of the app events
// defined. If the NCP initiated a rejoin it will record this value internally for
// retrieval by ezspGetValue(VALUE_REAL_REJOIN_REASON).
#define EZSP_VALUE_ID_LAST_REJOIN_REASON 0x13
// The next ZigBee sequence number.
#define EZSP_VALUE_ID_NEXT_ZIGBEE_SEQUENCE_NUMBER 0x14
// CCA energy detect threshold for radio.
#define EZSP_VALUE_ID_CCA_THRESHOLD 0x15
// The threshold value for a counter
#define EZSP_VALUE_ID_SET_COUNTER_THRESHOLD 0x17
// Resets all counters thresholds to 0xFF
#define EZSP_VALUE_ID_RESET_COUNTER_THRESHOLDS 0x18
// Clears all the counters
#define EZSP_VALUE_ID_CLEAR_COUNTERS 0x19
// The node's new certificate signed by the CA.
#define EZSP_EZSP_VALUE_ID_CERTIFICATE_283K1 0x1A
// The Certificate Authority's public key.
#define EZSP_EZSP_VALUE_ID_PUBLIC_KEY_283K1 0x1B
// The node's new static private key.
#define EZSP_EZSP_VALUE_ID_PRIVATE_KEY_283K1 0x1C
// The NWK layer security frame counter value
#define EZSP_VALUE_ID_NWK_FRAME_COUNTER 0x23
// The APS layer security frame counter value
#define EZSP_VALUE_ID_APS_FRAME_COUNTER 0x24
// Sets the device type to use on the next rejoin using device type
#define EZSP_VALUE_ID_RETRY_DEVICE_TYPE 0x25
// Setting this byte enables R21 behavior on the NCP.
#define EZSP_VALUE_ID_ENABLE_R21_BEHAVIOR 0x29
// Configure the antenna mode(0-primary,1-secondary,2- toggle on tx ack fail).
#define EZSP_VALUE_ID_ANTENNA_MODE 0x30
// Enable or disable packet traffic arbitration.
#define EZSP_VALUE_ID_ENABLE_PTA 0x31
// Set packet traffic arbitration configuration options.
#define EZSP_VALUE_ID_PTA_OPTIONS 0x32
// Configure manufacturing library options(0-non-CSMA transmits,1-CSMA transmits).
#define EZSP_VALUE_ID_MFGLIB_OPTIONS 0x33
// Sets the flag to use either negotiated power by link power delta (LPD) or fixed
// power value provided by user while forming/joining a network for packet
// transmissions on subghz interface. This is mainly for testing purposes.
#define EZSP_VALUE_ID_USE_NEGOTIATED_POWER_BY_LPD 0x34
// Set packet traffic arbitration configuration PWM options.
#define EZSP_VALUE_ID_PTA_PWM_OPTIONS 0x35
// Set packet traffic arbitration directional priority pulse width in microseconds.
#define EZSP_VALUE_ID_PTA_DIRECTIONAL_PRIORITY_PULSE_WIDTH 0x36
// Set packet traffic arbitration phy select timeout(ms)
#define EZSP_VALUE_ID_PTA_PHY_SELECT_TIMEOUT 0x37
// Configure the RX antenna mode: (0-do not switch; 1- primary; 2-secondary;
// 3-RX antenna diversity).
#define EZSP_VALUE_ID_ANTENNA_RX_MODE 0x38
// Configure the timeout to wait for the network key before failing a join.
#define EZSP_VALUE_ID_NWK_KEY_TIMEOUT 0x39
// The number of failed CSMA attempts due to failed CCA made by the MAC before
// continuing transmission with CCA disabled. This is the same as calling the
// emberForceTxAfterFailedCca(uint8_t csmaAttempts) API. A value of 0 disables the
// feature
#define EZSP_VALUE_ID_FORCE_TX_AFTER_FAILED_CCA_ATTEMPTS 0x3A
// The length of time, in seconds, that a trust center will store a transient link
// key that a device can use to join its network. A transient key is added with a
// call to emberAddTransientLinkKey. After the transient key is added, it will be
// removed once this amount of time has passed. A joining device will not be able to
// use that key to join until it is added again on the trust center. The default
// value is 300 seconds (5 minutes).
#define EZSP_VALUE_ID_TRANSIENT_KEY_TIMEOUT_S 0x3B
// Cumulative energy usage metric since the last value reset of the coulomb counter
// plugin. Setting this value will reset the coulomb counter.
#define EZSP_VALUE_ID_COULOMB_COUNTER_USAGE 0x3C
// When scanning, configure the maximum number of beacons to store in cache. Each
// beacon consumes one packet buffer in RAM.
#define EZSP_VALUE_ID_MAX_BEACONS_TO_STORE 0x3D
// Set the mask to filter out unacceptable child timeout options on a router.
#define EZSP_VALUE_ID_END_DEVICE_TIMEOUT_OPTIONS_MASK 0x3E
// The end device keep alive mode supported by the parent.
#define EZSP_VALUE_ID_END_DEVICE_KEEP_ALIVE_SUPPORT_MODE 0x3F
// Sets the mask that controls which pins will have their GPIO configuration and
// output values set to their power-up and power-down values when the NCP powers
// the radio up and down.
#define EZSP_VALUE_ID_GPIO_RADIO_POWER_MASK 0x40
// Return the active radio config.
#define EZSP_VALUE_ID_ACTIVE_RADIO_CONFIG 0x41

// Identifies a policy.

// Controls trust center behavior.
#define EZSP_POLICY_ID_TRUST_CENTER_POLICY 0x00
// Controls how external binding modification requests are handled.
#define EZSP_POLICY_ID_BINDING_MODIFICATION_POLICY 0x01
// Controls whether the Host supplies unicast replies.
#define EZSP_POLICY_ID_UNICAST_REPLIES_POLICY 0x02
// Controls whether pollHandler callbacks are generated.
#define EZSP_POLICY_ID_POLL_HANDLER_POLICY 0x03
// Controls whether the message contents are included in the
// messageSentHandler callback.
#define EZSP_POLICY_ID_MESSAGE_CONTENTS_IN_CALLBACK_POLICY 0x04
// Controls whether the Trust Center will respond to Trust Center link key
// requests.
#define EZSP_POLICY_ID_TC_KEY_REQUEST_POLICY 0x05
// Controls whether the Trust Center will respond to application link key
// requests.
#define EZSP_POLICY_ID_APP_KEY_REQUEST_POLICY 0x06
// Controls whether ZigBee packets that appear invalid are automatically
// dropped by the stack. A counter will be incremented when this occurs.
#define EZSP_POLICY_ID_PACKET_VALIDATE_LIBRARY_POLICY 0x07
// Controls whether the stack will process ZLL messages.
#define EZSP_POLICY_ID_ZLL_POLICY 0x08
// Controls whether Trust Center (insecure) rejoins for devices using the well-known
// link key are accepted. If rejoining using the well-known key is allowed, it is
// disabled again after emAllowTcRejoinsUsingWellKnownKeyTimeoutSec seconds.
#define EZSP_POLICY_ID_TC_REJOINS_USING_WELL_KNOWN_KEY_POLICY 0x09

// EZSP Decision bitmask //

// Disallow joins and rejoins.
#define EZSP_DECISION_BITMASK_DEFAULT_CONFIGURATION 0x0000
// Send the network key to all joining devices.
#define EZSP_DECISION_BITMASK_ALLOW_JOINS 0x0001
// Send the network key to all rejoining devices.
#define EZSP_DECISION_BITMASK_ALLOW_UNSECURED_REJOINS 0x0002
// Send the network key in the clear.
#define EZSP_DECISION_BITMASK_SEND_KEY_IN_CLEAR 0x0004
// Do nothing for unsecured rejoins.
#define EZSP_DECISION_BITMASK_IGNORE_UNSECURED_REJOINS 0x0008
// Allow joins if there is an entry in the transient key table.
#define EZSP_DECISION_BITMASK_JOINS_USE_INSTALL_CODE_KEY 0x0010
// Delay sending the network key to a new joining device.
#define EZSP_DECISION_BITMASK_DEFER_JOINS 0x0020

// Success.
#define EZSP_STATUS_SUCCESS 0x00
// Fatal error.
#define EZSP_STATUS_SPI_ERR_FATAL 0x10
// The Response frame of the current transaction indicates the NCP has
// reset.
#define EZSP_STATUS_SPI_ERR_NCP_RESET 0x11
// The NCP is reporting that the Command frame of the current transaction is
// oversized (the length byte is too large).
#define EZSP_STATUS_SPI_ERR_OVERSIZED_FRAME 0x12
// The Response frame of the current transaction indicates the previous
// transaction was aborted (nSSEL deasserted too soon).
#define EZSP_STATUS_SPI_ERR_ABORTED_TRANSACTION 0x13
// The Response frame of the current transaction indicates the frame
// terminator is missing from the Command frame.
#define EZSP_STATUS_SPI_ERR_MISSING_FRAME_TERMINATOR 0x14
// The NCP has not provided a Response within the time limit defined by
// WAIT_SECTION_TIMEOUT.
#define EZSP_STATUS_SPI_ERR_WAIT_SECTION_TIMEOUT 0x15
// The Response frame from the NCP is missing the frame terminator.
#define EZSP_STATUS_SPI_ERR_NO_FRAME_TERMINATOR 0x16
// The Host attempted to send an oversized Command (the length byte is too
// large) and the AVR's spi-protocol.c blocked the transmission.
#define EZSP_STATUS_SPI_ERR_COMMAND_OVERSIZED 0x17
// The NCP attempted to send an oversized Response (the length byte is too
// large) and the AVR's spi-protocol.c blocked the reception.
#define EZSP_STATUS_SPI_ERR_RESPONSE_OVERSIZED 0x18
// The Host has sent the Command and is still waiting for the NCP to send a
// Response.
#define EZSP_STATUS_SPI_WAITING_FOR_RESPONSE 0x19
// The NCP has not asserted nHOST_INT within the time limit defined by
// WAKE_HANDSHAKE_TIMEOUT.
#define EZSP_STATUS_SPI_ERR_HANDSHAKE_TIMEOUT 0x1A
// The NCP has not asserted nHOST_INT after an NCP reset within the time
// limit defined by STARTUP_TIMEOUT.
#define EZSP_STATUS_SPI_ERR_STARTUP_TIMEOUT 0x1B
// The Host attempted to verify the SPI Protocol activity and version
// number, and the verification failed.
#define EZSP_STATUS_SPI_ERR_STARTUP_FAIL 0x1C
// The Host has sent a command with a SPI Byte that is unsupported by the
// current mode the NCP is operating in.
#define EZSP_STATUS_SPI_ERR_UNSUPPORTED_SPI_COMMAND 0x1D
// Operation not yet complete.
#define EZSP_STATUS_ASH_IN_PROGRESS 0x20
// Fatal error detected by host.
#define EZSP_STATUS_HOST_FATAL_ERROR 0x21
// Fatal error detected by NCP.
#define EZSP_STATUS_ASH_NCP_FATAL_ERROR 0x22
// Tried to send DATA frame too long.
#define EZSP_STATUS_DATA_FRAME_TOO_LONG 0x23
// Tried to send DATA frame too short.
#define EZSP_STATUS_DATA_FRAME_TOO_SHORT 0x24
// No space for tx'ed DATA frame.
#define EZSP_STATUS_NO_TX_SPACE 0x25
// No space for rec'd DATA frame.
#define EZSP_STATUS_NO_RX_SPACE 0x26
// No receive data available.
#define EZSP_STATUS_NO_RX_DATA 0x27
// Not in Connected state.
#define EZSP_STATUS_NOT_CONNECTED 0x28
// The NCP received a command before the EZSP version had been set.
#define EZSP_STATUS_ERROR_VERSION_NOT_SET 0x30
// The NCP received a command containing an unsupported frame ID.
#define EZSP_STATUS_ERROR_INVALID_FRAME_ID 0x31
// The direction flag in the frame control field was incorrect.
#define EZSP_STATUS_ERROR_WRONG_DIRECTION 0x32
// The truncated flag in the frame control field was set, indicating there
// was not enough memory available to complete the response or that the
// response would have exceeded the maximum EZSP frame length.
#define EZSP_STATUS_ERROR_TRUNCATED 0x33
// The overflow flag in the frame control field was set, indicating one or
// more callbacks occurred since the previous response and there was not
// enough memory available to report them to the Host.
#define EZSP_STATUS_ERROR_OVERFLOW 0x34
// Insufficient memory was available.
#define EZSP_STATUS_ERROR_OUT_OF_MEMORY 0x35
// The value was out of bounds.
#define EZSP_STATUS_ERROR_INVALID_VALUE 0x36
// The configuration id was not recognized.
#define EZSP_STATUS_ERROR_INVALID_ID 0x37
// Configuration values can no longer be modified.
#define EZSP_STATUS_ERROR_INVALID_CALL 0x38
// The NCP failed to respond to a command.
#define EZSP_STATUS_ERROR_NO_RESPONSE 0x39
// The length of the command exceeded the maximum EZSP frame length.
#define EZSP_STATUS_ERROR_COMMAND_TOO_LONG 0x40
// The UART receive queue was full causing a callback response to be dropped.
#define EZSP_STATUS_ERROR_QUEUE_FULL 0x41
// The command has been filtered out by NCP.
#define EZSP_STATUS_ERROR_COMMAND_FILTERED 0x42
// EZSP Security Key is already set
#define EZSP_STATUS_ERROR_SECURITY_KEY_ALREADY_SET 0x43
// EZSP Security Type is invalid
#define EZSP_STATUS_ERROR_SECURITY_TYPE_INVALID 0x44
// EZSP Security Parameters are invalid
#define EZSP_STATUS_ERROR_SECURITY_PARAMETERS_INVALID 0x45
// EZSP Security Parameters are already set
#define EZSP_STATUS_ERROR_SECURITY_PARAMETERS_ALREADY_SET 0x46
// EZSP Security Key is not set
#define EZSP_STATUS_ERROR_SECURITY_KEY_NOT_SET 0x47
// EZSP Security Parameters are not set
#define EZSP_STATUS_ERROR_SECURITY_PARAMETERS_NOT_SET 0x48
// Received frame with unsupported control byte
#define EZSP_STATUS_ERROR_UNSUPPORTED_CONTROL 0x49
// Received frame is unsecure, when security is established
#define EZSP_STATUS_ERROR_UNSECURE_FRAME 0x4A
// Incompatible ASH version
#define EZSP_STATUS_ASH_ERROR_VERSION 0x50
// Exceeded max ACK timeouts
#define EZSP_STATUS_ASH_ERROR_TIMEOUTS 0x51
// Timed out waiting for RSTACK
#define EZSP_STATUS_ASH_ERROR_RESET_FAIL 0x52
// Unexpected ncp reset
#define EZSP_STATUS_ASH_ERROR_NCP_RESET 0x53
// Serial port initialization failed
#define EZSP_STATUS_ERROR_SERIAL_INIT 0x54
// Invalid ncp processor type
#define EZSP_STATUS_ASH_ERROR_NCP_TYPE 0x55
// Invalid ncp reset method
#define EZSP_STATUS_ASH_ERROR_RESET_METHOD 0x56
// XON/XOFF not supported by host driver
#define EZSP_STATUS_ASH_ERROR_XON_XOFF 0x57
// ASH protocol started
#define EZSP_STATUS_ASH_STARTED 0x70
// ASH protocol connected
#define EZSP_STATUS_ASH_CONNECTED 0x71
// ASH protocol disconnected
#define EZSP_STATUS_ASH_DISCONNECTED 0x72
// Timer expired waiting for ack
#define EZSP_STATUS_ASH_ACK_TIMEOUT 0x73
// Frame in progress cancelled
#define EZSP_STATUS_ASH_CANCELLED 0x74
// Received frame out of sequence
#define EZSP_STATUS_ASH_OUT_OF_SEQUENCE 0x75
// Received frame with CRC error
#define EZSP_STATUS_ASH_BAD_CRC 0x76
// Received frame with comm error
#define EZSP_STATUS_ASH_COMM_ERROR 0x77
// Received frame with bad ackNum
#define EZSP_STATUS_ASH_BAD_ACKNUM 0x78
// Received frame shorter than minimum
#define EZSP_STATUS_ASH_TOO_SHORT 0x79
// Received frame longer than maximum
#define EZSP_STATUS_ASH_TOO_LONG 0x7A
// Received frame with illegal control byte
#define EZSP_STATUS_ASH_BAD_CONTROL 0x7B
// Received frame with illegal length for its type
#define EZSP_STATUS_ASH_BAD_LENGTH 0x7C
// Received ASH Ack
#define EZSP_STATUS_ASH_ACK_RECEIVED 0x7D
// Sent ASH Ack
#define EZSP_STATUS_ASH_ACK_SENT 0x7E
// Received ASH Nak
#define EZSP_STATUS_ASH_NAK_RECEIVED 0x7F
// Sent ASH Nak
#define EZSP_STATUS_ASH_NAK_SENT 0x80
// Received ASH RST
#define EZSP_STATUS_ASH_RST_RECEIVED 0x81
// Sent ASH RST
#define EZSP_STATUS_ASH_RST_SENT 0x82
// ASH Status
#define EZSP_STATUS_ASH_STATUS 0x83
// ASH TX
#define EZSP_STATUS_ASH_TX 0x84
// ASH RX
#define EZSP_STATUS_ASH_RX 0x85
// Failed to connect to CPC daemon or failed to open CPC endpoint
#define EZSP_STATUS_CPC_ERROR_INIT 0x86
// No reset or error
#define EZSP_STATUS_NO_ERROR 0xFF


// Identifies a value.
// ZEzspValue values

// The contents of the node data stack token.
#define EZSP_VALUE_TOKEN_STACK_NODE_DATA 0x00
// The types of MAC passthrough messages that the host wishes to receive.
#define EZSP_VALUE_MAC_PASSTHROUGH_FLAGS 0x01
// The source address used to filter legacy EmberNet messages when the
// MAC_PASSTHROUGH_EMBERNET_SOURCE flag is set in VALUE_MAC_PASSTHROUGH_FLAGS.
#define EZSP_VALUE_EMBERNET_PASSTHROUGH_SOURCE_ADDRESS 0x02
// The number of available message buffers.
#define EZSP_VALUE_FREE_BUFFERS 0x03
// Selects sending synchronous callbacks in ezsp-uart.
#define EZSP_VALUE_UART_SYNCH_CALLBACKS 0x04
// The maximum incoming transfer size for the local node.
#define EZSP_VALUE_MAXIMUM_INCOMING_TRANSFER_SIZE 0x05
// The maximum outgoing transfer size for the local node.
#define EZSP_VALUE_MAXIMUM_OUTGOING_TRANSFER_SIZE 0x06
// A boolean indicating whether stack tokens are written to persistent
// storage as they change.
#define EZSP_VALUE_STACK_TOKEN_WRITING 0x07
// A read-only value indicating whether the stack is currently performing a rejoin.
#define EZSP_VALUE_STACK_IS_PERFORMING_REJOIN 0x08
// A list of EmberMacFilterMatchData values.
#define EZSP_VALUE_MAC_FILTER_LIST 0x09
// The Ember Extended Security Bitmask.
#define EZSP_VALUE_EXTENDED_SECURITY_BITMASK 0x0A
// The node short ID.
#define EZSP_VALUE_NODE_SHORT_ID 0x0B
// The descriptor capability of the local node.
#define EZSP_VALUE_DESCRIPTOR_CAPABILITY 0x0C
// The stack device request sequence number of the local node.
#define EZSP_VALUE_STACK_DEVICE_REQUEST_SEQUENCE_NUMBER 0x0D
// Enable or disable radio hold-off.
#define EZSP_VALUE_RADIO_HOLD_OFF 0x0E
// The flags field associated with the endpoint data.
#define EZSP_VALUE_ENDPOINT_FLAGS 0x0F
// Enable/disable the Mfg security config key settings.
#define EZSP_VALUE_MFG_SECURITY_CONFIG 0x10
// Retrieves the version information from the stack on the NCP.
#define EZSP_VALUE_VERSION_INFO 0x11
// This will get/set the rejoin reason noted by the host for a subsequent call to
// emberFindAndRejoinNetwork(). After a call to emberFindAndRejoinNetwork() the
// host's rejoin reason will be set to REJOIN_REASON_NONE. The NCP will store the
// rejoin reason used by the call to emberFindAndRejoinNetwork()
#define EZSP_VALUE_NEXT_HOST_REJOIN_REASON 0x12
// This is the reason that the last rejoin took place. This value may only be
// retrieved, not set. The rejoin may have been initiated by the stack (NCP) or the
// application (host). If a host initiated a rejoin the reason will be set by default
// to REJOIN_DUE_TO_APP_EVENT_1. If the application wishes to denote its own rejoin
// reasons it can do so by calling ezspSetValue(VALUE_HOST_REJOIN_REASON,
// REJOIN_DUE_TO_APP_EVENT_X). X is a number corresponding to one of the app events
// defined. If the NCP initiated a rejoin it will record this value internally for
// retrieval by ezspGetValue(VALUE_REAL_REJOIN_REASON).
#define EZSP_VALUE_LAST_REJOIN_REASON 0x13
// The next ZigBee sequence number.
#define EZSP_VALUE_NEXT_ZIGBEE_SEQUENCE_NUMBER 0x14
// CCA energy detect threshold for radio.
#define EZSP_VALUE_CCA_THRESHOLD 0x15
// The threshold value for a counter
#define EZSP_VALUE_SET_COUNTER_THRESHOLD 0x17
// Resets all counters thresholds to 0xFF
#define EZSP_VALUE_RESET_COUNTER_THRESHOLDS 0x18
// Clears all the counters
#define EZSP_VALUE_CLEAR_COUNTERS 0x19
// The node's new certificate signed by the CA.
#define EZSP_EZSP_VALUE_CERTIFICATE_283K1 0x1A
// The Certificate Authority's public key.
#define EZSP_EZSP_VALUE_PUBLIC_KEY_283K1 0x1B
// The node's new static private key.
#define EZSP_EZSP_VALUE_PRIVATE_KEY_283K1 0x1C
// The NWK layer security frame counter value
#define EZSP_VALUE_NWK_FRAME_COUNTER 0x23
// The APS layer security frame counter value
#define EZSP_VALUE_APS_FRAME_COUNTER 0x24
// Sets the device type to use on the next rejoin using device type
#define EZSP_VALUE_RETRY_DEVICE_TYPE 0x25
// Setting this byte enables R21 behavior on the NCP.
#define EZSP_VALUE_ENABLE_R21_BEHAVIOR 0x29
// Configure the antenna mode(0-primary,1-secondary,2- toggle on tx ack fail).
#define EZSP_VALUE_ANTENNA_MODE 0x30
// Enable or disable packet traffic arbitration.
#define EZSP_VALUE_ENABLE_PTA 0x31
// Set packet traffic arbitration configuration options.
#define EZSP_VALUE_PTA_OPTIONS 0x32
// Configure manufacturing library options(0-non-CSMA transmits,1-CSMA transmits).
#define EZSP_VALUE_MFGLIB_OPTIONS 0x33
// Sets the flag to use either negotiated power by link power delta (LPD) or fixed
// power value provided by user while forming/joining a network for packet
// transmissions on subghz interface. This is mainly for testing purposes.
#define EZSP_VALUE_USE_NEGOTIATED_POWER_BY_LPD 0x34
// Set packet traffic arbitration configuration PWM options.
#define EZSP_VALUE_PTA_PWM_OPTIONS 0x35
// Set packet traffic arbitration directional priority pulse width in microseconds.
#define EZSP_VALUE_PTA_DIRECTIONAL_PRIORITY_PULSE_WIDTH 0x36
// Set packet traffic arbitration phy select timeout(ms)
#define EZSP_VALUE_PTA_PHY_SELECT_TIMEOUT 0x37
// Configure the RX antenna mode: (0-do not switch; 1- primary; 2-secondary;
// 3-RX antenna diversity).
#define EZSP_VALUE_ANTENNA_RX_MODE 0x38
// Configure the timeout to wait for the network key before failing a join.
#define EZSP_VALUE_NWK_KEY_TIMEOUT 0x39
// The number of failed CSMA attempts due to failed CCA made by the MAC before
// continuing transmission with CCA disabled. This is the same as calling the
// emberForceTxAfterFailedCca(uint8_t csmaAttempts) API. A value of 0 disables the
// feature
#define EZSP_VALUE_FORCE_TX_AFTER_FAILED_CCA_ATTEMPTS 0x3A
// The length of time, in seconds, that a trust center will store a transient link
// key that a device can use to join its network. A transient key is added with a
// call to emberAddTransientLinkKey. After the transient key is added, it will be
// removed once this amount of time has passed. A joining device will not be able to
// use that key to join until it is added again on the trust center. The default
// value is 300 seconds (5 minutes).
#define EZSP_VALUE_TRANSIENT_KEY_TIMEOUT_S 0x3B
// Cumulative energy usage metric since the last value reset of the coulomb counter
// plugin. Setting this value will reset the coulomb counter.
#define EZSP_VALUE_COULOMB_COUNTER_USAGE 0x3C
// When scanning, configure the maximum number of beacons to store in cache. Each
// beacon consumes one packet buffer in RAM.
#define EZSP_VALUE_MAX_BEACONS_TO_STORE 0x3D
// Set the mask to filter out unacceptable child timeout options on a router.
#define EZSP_VALUE_END_DEVICE_TIMEOUT_OPTIONS_MASK 0x3E
// The end device keep alive mode supported by the parent.
#define EZSP_VALUE_END_DEVICE_KEEP_ALIVE_SUPPORT_MODE 0x3F
// Return the active radio config.
#define EZSP_VALUE_ACTIVE_RADIO_CONFIG 0x41
// Timeout in milliseconds to store entries in the transient device table.
// If the devices are not authenticated before the timeout, the entry shall be
// purged.
#define EZSP_VALUE_TRANSIENT_DEVICE_TIMEOUT 0x43



// EZSP Decision bitmask //

// Disallow joins and rejoins.
#define EZSP_DECISION_DEFAULT_CONFIGURATION_MASK 0x0000
// Send the network key to all joining devices.
#define EZSP_DECISION_ALLOW_JOINS_MASK 0x0001
// Send the network key to all rejoining devices.
#define EZSP_DECISION_ALLOW_UNSECURED_REJOINS_MASK 0x0002
// Send the network key in the clear.
#define EZSP_DECISION_SEND_KEY_IN_CLEAR_MASK 0x0004
// Do nothing for unsecured rejoins.
#define EZSP_DECISION_IGNORE_UNSECURED_REJOINS_MASK 0x0008
// Allow joins if there is an entry in the transient key table.
#define EZSP_DECISION_JOINS_USE_INSTALL_CODE_KEY_MASK 0x0010
// Delay sending the network key to a new joining device.
#define EZSP_DECISION_DEFER_JOINS_MASK 0x0020


// EZSP Deciosion Id

// BINDING_MODIFICATION_POLICY default decision. Do not allow the local
// binding table to be changed by remote nodes.
#define EZSP_DECISION_ID_DISALLOW_BINDING_MODIFICATION 0x10
// BINDING_MODIFICATION_POLICY decision.  Allow remote nodes to change
// the local binding table.
#define EZSP_DECISION_ID_ALLOW_BINDING_MODIFICATION 0x11
// BINDING_MODIFICATION_POLICY decision.  Allows remote nodes to set local
// binding entries only if the entries correspond to endpoints defined on
// the device, and for output clusters bound to those endpoints.
#define EZSP_DECISION_ID_CHECK_BINDING_MODIFICATIONS_ARE_VALID_ENDPOINT_CLUSTERS 0x12
// UNICAST_REPLIES_POLICY default decision.  The NCP will automatically send
// an empty reply (containing no payload) for every unicast received.
#define EZSP_DECISION_ID_HOST_WILL_NOT_SUPPLY_REPLY 0x20
// UNICAST_REPLIES_POLICY decision. The NCP will only send a reply if it
// receives a sendReply command from the Host.
#define EZSP_DECISION_ID_HOST_WILL_SUPPLY_REPLY 0x21
// POLL_HANDLER_POLICY default decision. Do not inform the Host when a child polls.
#define EZSP_DECISION_ID_POLL_HANDLER_IGNORE 0x30
// POLL_HANDLER_POLICY decision. Generate a pollHandler callback when a child polls.
#define EZSP_DECISION_ID_POLL_HANDLER_CALLBACK 0x31
// MESSAGE_CONTENTS_IN_CALLBACK_POLICY default decision. Include only the
// message tag in the messageSentHandler callback.
#define EZSP_DECISION_ID_MESSAGE_TAG_ONLY_IN_CALLBACK 0x40
// MESSAGE_CONTENTS_IN_CALLBACK_POLICY decision. Include both the message
// tag and the message contents in the messageSentHandler callback.
#define EZSP_DECISION_ID_MESSAGE_TAG_AND_CONTENTS_IN_CALLBACK 0x41
// TC_KEY_REQUEST_POLICY decision. When the Trust Center receives a request
// for a Trust Center link key, it will be ignored.
#define EZSP_DECISION_ID_DENY_TC_KEY_REQUESTS 0x50
// TC_KEY_REQUEST_POLICY decision. When the Trust Center receives a request for a
// Trust Center link key, it will reply to it with the corresponding key.
#define EZSP_DECISION_ID_ALLOW_TC_KEY_REQUESTS_AND_SEND_CURRENT_KEY 0x51
// TC_KEY_REQUEST_POLICY decision. When the Trust Center receives a request
// for a Trust Center link key, it will generate a key to send to the joiner.
#define EZSP_DECISION_ID_ALLOW_TC_KEY_REQUEST_AND_GENERATE_NEW_KEY 0x52
// APP_KEY_REQUEST_POLICY decision. When the Trust Center receives a request
// for an application link key, it will be ignored.
#define EZSP_DECISION_ID_DENY_APP_KEY_REQUESTS 0x60
// APP_KEY_REQUEST_POLICY decision. When the Trust Center receives a request
// for an application link key, it will randomly generate a key and send it
// to both partners.
#define EZSP_DECISION_ID_ALLOW_APP_KEY_REQUESTS 0x61
// Indicates that packet validate library checks are enabled on the NCP.
#define EZSP_DECISION_ID_PACKET_VALIDATE_LIBRARY_CHECKS_ENABLED 0x62
// Indicates that packet validate library checks are NOT enabled on the NCP.
#define EZSP_DECISION_ID_PACKET_VALIDATE_LIBRARY_CHECKS_DISABLED 0x63

// Bitmask options //

// No options for Network Init
#define EMBER_NWK_INIT_NETWORK_INIT_NO_OPTIONS_MASK 0x0000
// Save parent info (node ID and EUI64) in a token during joining/rejoin,
// and restore on reboot.
#define EMBER_NWK_INIT_NETWORK_INIT_PARENT_INFO_IN_TOKEN_MASK 0x0001
// Send a rejoin request as an end device on reboot if parent information is
// persisted. ZB3 end devices must rejoin on reboot.
#define EMBER_NWK_INIT_NETWORK_INIT_END_DEVICE_REJOIN_ON_REBOOT_MASK 0x0002


// Network configuration for the desired radio interface for multi phy network. //

// Enable broadcast support on Routers
#define EMBER_MULTI_PHY_NWK_COBFIG_BROADCAST_SUPPORT 0x01


// Duty cycle states. //

// No Duty cycle tracking or metrics are taking place
#define EMBER_DUTY_CYCLE_TRACKING_OFF 0
// Duty Cycle is tracked and has not exceeded any thresholds.
#define EMBER_DUTY_CYCLE_LBT_NORMAL 1
// We have exceeded the limited threshold of our total duty cycle allotment.
#define EMBER_DUTY_CYCLE_LBT_LIMITED_THRESHOLD_REACHED 2
// We have exceeded the critical threshold of our total duty cycle allotment.
#define EMBER_DUTY_CYCLE_LBT_CRITICAL_THRESHOLD_REACHED 3
// We have reached the suspend limit and are blocking all outbound transmissions.
#define EMBER_DUTY_CYCLE_LBT_SUSPEND_LIMIT_REACHED 4


// Radio power mode. //

// The radio receiver is switched on.
#define EMBER_RADIO_POWER_MODE_RX_ON 0
// The radio receiver is switched off.
#define EMBER_RADIO_POWER_MODE_OFF 1


// Entropy sources. //

// Entropy source error
#define EMBER_ENTROPY_SOURCE_ENTROPY_SOURCE_ERROR 0
// Entropy source is the radio.
#define EMBER_ENTROPY_SOURCE_ENTROPY_SOURCE_RADIO 1
// Entropy source is the TRNG powered by mbed TLS.
#define EMBER_ENTROPY_SOURCE_ENTROPY_SOURCE_MBEDTLS_TRNG 2
// Entropy source is powered by mbed TLS, the source is not TRNG.
#define EMBER_ENTROPY_SOURCE_ENTROPY_SOURCE_MBEDTLS 3


// Flags for controlling which incoming ZDO requests are passed to the        //
// application. To see if the application is required to send a ZDO response  //
// to an incoming message, the application must check the APS options         //
// bitfield within the incomingMessageHandler callback to see if the          //
// APS_OPTION_ZDO_RESPONSE_REQUIRED flag is set.                              //

// Set this flag in order to receive supported ZDO request messages via the
// incomingMessageHandler callback. A supported ZDO request is one that is
// handled by the EmberZNet stack. The stack will continue to handle the
// request and send the appropriate ZDO response even if this configuration
// option is enabled.
#define EMBER_APP_RECEIVES_SUPPORTED_ZDO_REQUESTS 0x01
// Set this flag in order to receive unsupported ZDO request messages via
// the incomingMessageHandler callback. An unsupported ZDO request is one
// that is not handled by the EmberZNet stack, other than to send a 'not
// supported' ZDO response. If this configuration option is enabled, the
// stack will no longer send any ZDO response, and it is the application's
// responsibility to do so.
#define EMBER_APP_HANDLES_UNSUPPORTED_ZDO_REQUESTS 0x02
// Set this flag in order to receive the following ZDO request messages via
// the incomingMessageHandler callback: SIMPLE_DESCRIPTOR_REQUEST,
// MATCH_DESCRIPTORS_REQUEST, and ACTIVE_ENDPOINTS_REQUEST. If this
// configuration option is enabled, the stack will no longer send any ZDO
// response for these requests, and it is the application's responsibility
// to do so.
#define EMBER_APP_HANDLES_ZDO_ENDPOINT_REQUESTS 0x04
// Set this flag in order to receive the following ZDO request messages via
// the incomingMessageHandler callback: BINDING_TABLE_REQUEST, BIND_REQUEST,
// and UNBIND_REQUEST. If this configuration option is enabled, the stack
// will no longer send any ZDO response for these requests, and it is the
// application's responsibility to do so.
#define EMBER_APP_HANDLES_ZDO_BINDING_REQUESTS 0x08


// Manufacturing token IDs used by efGetMfgToken(). //

// Custom version (2 bytes).
#define EZSP_MFG_CUSTOM_VERSION 0x00
// Manufacturing string (16 bytes).
#define EZSP_MFG_STRING 0x01
// Board name (16 bytes).
#define EZSP_MFG_BOARD_NAME 0x02
// Manufacturing ID (2 bytes).
#define EZSP_MFG_MANUF_ID 0x03
// Radio configuration (2 bytes).
#define EZSP_MFG_PHY_CONFIG 0x04
// Bootload AES key (16 bytes).
#define EZSP_MFG_BOOTLOAD_AES_KEY 0x05
// ASH configuration (40 bytes).
#define EZSP_MFG_ASH_CONFIG 0x06
// EZSP storage (8 bytes).
#define EZSP_MFG_STORAGE 0x07
// Radio calibration data (64 bytes). 4 bytes are stored for each of the 16
// channels. This token is not stored in the Flash Information Area. It is
// updated by the stack each time a calibration is performed.
#define EZSP_STACK_CAL_DATA 0x08
// Certificate Based Key Exchange (CBKE) data (92 bytes).
#define EZSP_MFG_CBKE_DATA 0x09
// Installation code (20 bytes).
#define EZSP_MFG_INSTALLATION_CODE 0x0A
// Radio channel filter calibration data (1 byte). This token is not stored
// in the Flash Information Area. It is updated by the stack each time a
// calibration is performed.
#define EZSP_STACK_CAL_FILTER 0x0B
// Custom EUI64 MAC address (8 bytes).
#define EZSP_MFG_CUSTOM_EUI_64 0x0C
// CTUNE value (2 byte).
#define EZSP_MFG_CTUNE 0x0D

// Return type for stack functions.

// The generic 'no error' message.
#define EMBER_STATUS_SUCCESS 0x00
// The generic 'fatal error' message.
#define EMBER_STATUS_ERR_FATAL 0x01
// An invalid value was passed as an argument to a function
#define EMBER_STATUS_BAD_ARGUMENT 0x02
// The manufacturing and stack token format in nonvolatile memory is
// different than what the stack expects (returned at initialization).
#define EMBER_STATUS_EEPROM_MFG_STACK_VERSION_MISMATCH 0x04
// The static memory definitions in ember-staticmemory.h are incompatible
// with this stack version.
#define EMBER_STATUS_INCOMPATIBLE_STATIC_MEMORY_DEFINITIONS 0x05
// The manufacturing token format in non-volatile memory is different than
// what the stack expects (returned at initialization).
#define EMBER_STATUS_EEPROM_MFG_VERSION_MISMATCH 0x06
// The stack token format in non-volatile memory is different than what the
// stack expects (returned at initialization).
#define EMBER_STATUS_EEPROM_STACK_VERSION_MISMATCH 0x07
// There are no more buffers.
#define EMBER_STATUS_NO_BUFFERS 0x18
// Specified an invalid baud rate.
#define EMBER_STATUS_SERIAL_INVALID_BAUD_RATE 0x20
// Specified an invalid serial port.
#define EMBER_STATUS_SERIAL_INVALID_PORT 0x21
// Tried to send too much data.
#define EMBER_STATUS_SERIAL_TX_OVERFLOW 0x22
// There was not enough space to store a received character and the
// character was dropped.
#define EMBER_STATUS_SERIAL_RX_OVERFLOW 0x23
// Detected a UART framing error.
#define EMBER_STATUS_SERIAL_RX_FRAME_ERROR 0x24
// Detected a UART parity error.
#define EMBER_STATUS_SERIAL_RX_PARITY_ERROR 0x25
// There is no received data to process.
#define EMBER_STATUS_SERIAL_RX_EMPTY 0x26
// The receive interrupt was not handled in time, and a character was
// dropped.
#define EMBER_STATUS_SERIAL_RX_OVERRUN_ERROR 0x27
// The MAC transmit queue is full.
#define EMBER_STATUS_MAC_TRANSMIT_QUEUE_FULL 0x39
// MAC header FCR error on receive.
#define EMBER_STATUS_MAC_UNKNOWN_HEADER_TYPE 0x3A
// The MAC can't complete this task because it is scanning.
#define EMBER_STATUS_MAC_SCANNING 0x3D
// No pending data exists for device doing a data poll.
#define EMBER_STATUS_MAC_NO_DATA 0x31
// Attempt to scan when we are joined to a network.
#define EMBER_STATUS_MAC_JOINED_NETWORK 0x32
// Scan duration must be 0 to 14 inclusive. Attempt was made to scan with an
// incorrect duration value.
#define EMBER_STATUS_MAC_BAD_SCAN_DURATION 0x33
// emberStartScan was called with an incorrect scan type.
#define EMBER_STATUS_MAC_INCORRECT_SCAN_TYPE 0x34
// emberStartScan was called with an invalid channel mask.
#define EMBER_STATUS_MAC_INVALID_CHANNEL_MASK 0x35
// Failed to scan current channel because we were unable to transmit the
// relevant MAC command.
#define EMBER_STATUS_MAC_COMMAND_TRANSMIT_FAILURE 0x36
// We expected to receive an ACK following the transmission, but the MAC
// level ACK was never received.
#define EMBER_STATUS_MAC_NO_ACK_RECEIVED 0x40
// Indirect data message timed out before polled.
#define EMBER_STATUS_MAC_INDIRECT_TIMEOUT 0x42
// The Simulated EEPROM is telling the application that there is at least
// one flash page to be erased.  The GREEN status means the current page has
// not filled above the ERASE_CRITICAL_THRESHOLD.  The application should
// call the function halSimEepromErasePage when it can to erase a page.
#define EMBER_STATUS_SIM_EEPROM_ERASE_PAGE_GREEN 0x43
// The Simulated EEPROM is telling the application that there is at least
// one flash page to be erased.  The RED status means the current page has
// filled above the ERASE_CRITICAL_THRESHOLD. Due to the shrinking
// availability of write space, there is a danger of data loss. The
// application must call the function halSimEepromErasePage as soon as
// possible to erase a page.
#define EMBER_STATUS_SIM_EEPROM_ERASE_PAGE_RED 0x44
// The Simulated EEPROM has run out of room to write any new data and the
// data trying to be set has been lost. This error code is the result of
// ignoring the SIM_EEPROM_ERASE_PAGE_RED error code.  The application must
// call the function halSimEepromErasePage to make room for any further
// calls to set a token.
#define EMBER_STATUS_SIM_EEPROM_FULL 0x45
// A fatal error has occurred while trying to write data to the Flash. The
// target memory attempting to be programmed is already programmed. The
// flash write routines were asked to flip a bit from a 0 to 1, which is
// physically impossible and the write was therefore inhibited. The data in
// the flash cannot be trusted after this error.
#define EMBER_STATUS_ERR_FLASH_WRITE_INHIBITED 0x46
// A fatal error has occurred while trying to write data to the Flash and
// the write verification has failed. The data in the flash cannot be
// trusted after this error, and it is possible this error is the result of
// exceeding the life cycles of the flash.
#define EMBER_STATUS_ERR_FLASH_VERIFY_FAILED 0x47
// Attempt 1 to initialize the Simulated EEPROM has failed. This failure
// means the information already stored in Flash (or a lack thereof), is
// fatally incompatible with the token information compiled into the code
// image being run.
#define EMBER_STATUS_SIM_EEPROM_INIT_1_FAILED 0x48
// Attempt 2 to initialize the Simulated EEPROM has failed. This failure
// means Attempt 1 failed, and the token system failed to properly reload
// default tokens and reset the Simulated EEPROM.
#define EMBER_STATUS_SIM_EEPROM_INIT_2_FAILED 0x49
// Attempt 3 to initialize the Simulated EEPROM has failed. This failure
// means one or both of the tokens TOKEN_MFG_NVDATA_VERSION or
// TOKEN_STACK_NVDATA_VERSION were incorrect and the token system failed to
// properly reload default tokens and reset the Simulated EEPROM.
#define EMBER_STATUS_SIM_EEPROM_INIT_3_FAILED 0x4A
// A fatal error has occurred while trying to write data to the flash,
// possibly due to write protection or an invalid address. The data in the
// flash cannot be trusted after this error, and it is possible this error
// is the result of exceeding the life cycles of the flash.
#define EMBER_STATUS_ERR_FLASH_PROG_FAIL 0x4B
// A fatal error has occurred while trying to erase flash, possibly due to
// write protection. The data in the flash cannot be trusted after this
// error, and it is possible this error is the result of exceeding the life
// cycles of the flash.
#define EMBER_STATUS_ERR_FLASH_ERASE_FAIL 0x4C
// The bootloader received an invalid message (failed attempt to go into
// bootloader).
#define EMBER_STATUS_ERR_BOOTLOADER_TRAP_TABLE_BAD 0x58
// Bootloader received an invalid message (failed attempt to go into
// bootloader).
#define EMBER_STATUS_ERR_BOOTLOADER_TRAP_UNKNOWN 0x59
// The bootloader cannot complete the bootload operation because either an
// image was not found or the image exceeded memory bounds.
#define EMBER_STATUS_ERR_BOOTLOADER_NO_IMAGE 0x5A
// The APS layer attempted to send or deliver a message, but it failed.
#define EMBER_STATUS_DELIVERY_FAILED 0x66
// This binding index is out of range of the current binding table.
#define EMBER_STATUS_BINDING_INDEX_OUT_OF_RANGE 0x69
// This address table index is out of range for the current address table.
#define EMBER_STATUS_ADDRESS_TABLE_INDEX_OUT_OF_RANGE 0x6A
// An invalid binding table index was given to a function.
#define EMBER_STATUS_INVALID_BINDING_INDEX 0x6C
// The API call is not allowed given the current state of the stack.
#define EMBER_STATUS_INVALID_CALL 0x70
// The link cost to a node is not known.
#define EMBER_STATUS_COST_NOT_KNOWN 0x71
// The maximum number of in-flight messages (i.e.
// APS_UNICAST_MESSAGE_COUNT) has been reached.
#define EMBER_STATUS_MAX_MESSAGE_LIMIT_REACHED 0x72
// The message to be transmitted is too big to fit into a single over-the-
// air packet.
#define EMBER_STATUS_MESSAGE_TOO_LONG 0x74
// The application is trying to delete or overwrite a binding that is in
// use.
#define EMBER_STATUS_BINDING_IS_ACTIVE 0x75
// The application is trying to overwrite an address table entry that is in
// use.
#define EMBER_STATUS_ADDRESS_TABLE_ENTRY_IS_ACTIVE 0x76
// Conversion is complete.
#define EMBER_STATUS_ADC_CONVERSION_DONE 0x80
// Conversion cannot be done because a request is being processed.
#define EMBER_STATUS_ADC_CONVERSION_BUSY 0x81
// Conversion is deferred until the current request has been processed.
#define EMBER_STATUS_ADC_CONVERSION_DEFERRED 0x82
// No results are pending.
#define EMBER_STATUS_ADC_NO_CONVERSION_PENDING 0x84
// Sleeping (for a duration) has been abnormally interrupted and exited
// prematurely.
#define EMBER_STATUS_SLEEP_INTERRUPTED 0x85
// The transmit hardware buffer underflowed.
#define EMBER_STATUS_PHY_TX_UNDERFLOW 0x88
// The transmit hardware did not finish transmitting a packet.
#define EMBER_STATUS_PHY_TX_INCOMPLETE 0x89
// An unsupported channel setting was specified.
#define EMBER_STATUS_PHY_INVALID_CHANNEL 0x8A
// An unsupported power setting was specified.
#define EMBER_STATUS_PHY_INVALID_POWER 0x8B
// The packet cannot be transmitted because the physical MAC layer is
// currently transmitting a packet.  (This is used for the MAC backoff
// algorithm.) PHY_TX_CCA_FAIL 0x8D The transmit attempt failed because all
// CCA attempts indicated that the channel was busy
#define EMBER_STATUS_PHY_TX_BUSY 0x8C
// The transmit attempt failed because all CCA attempts indicated that the channel
// was busy
#define EMBER_STATUS_PHY_TX_CCA_FAIL 0x8D
// The software installed on the hardware doesn't recognize the hardware
// radio type.
#define EMBER_STATUS_PHY_OSCILLATOR_CHECK_FAILED 0x8E
// The expected ACK was received after the last transmission.
#define EMBER_STATUS_PHY_ACK_RECEIVED 0x8F
// The stack software has completed initialization and is ready to send and
// receive packets over the air.
#define EMBER_STATUS_NETWORK_UP 0x90
// The network is not operating.
#define EMBER_STATUS_NETWORK_DOWN 0x91
// An attempt to join a network failed.
#define EMBER_STATUS_JOIN_FAILED 0x94
// The chosen security level (the value of
// EMBER_SECURITY_LEVEL) is not supported by the stack.
#define EMBER_STATUS_INVALID_SECURITY_LEVEL 0x95
// After moving, a mobile node's attempt to re-establish contact with the
// network failed.
#define EMBER_STATUS_MOVE_FAILED 0x96
// An attempt to join as a router failed due to a ZigBee versus ZigBee Pro
// incompatibility. ZigBee devices joining ZigBee Pro networks (or vice
// versa) must join as End Devices, not Routers.
#define EMBER_STATUS_CANNOT_JOIN_AS_ROUTER 0x98
// The local node ID has changed. The application can obtain the new node ID
// by calling emberGetNodeId().
#define EMBER_STATUS_NODE_ID_CHANGED 0x99
// The local PAN ID has changed. The application can obtain the new PAN ID
// by calling emberGetPanId().
#define EMBER_STATUS_PAN_ID_CHANGED 0x9A
// The network has been opened for joining.
#define EMBER_STATUS_NETWORK_OPENED 0x9C
// The network has been closed for joining.
#define EMBER_STATUS_NETWORK_CLOSED 0x9D
// An attempt to join or rejoin the network failed because no router beacons
// could be heard by the joining node.
#define EMBER_STATUS_NO_BEACONS 0xAB
// An attempt was made to join a Secured Network using a pre-configured key,
// but the Trust Center sent back a Network Key in-the-clear when an
// encrypted Network Key was required.
#define EMBER_STATUS_RECEIVED_KEY_IN_THE_CLEAR 0xAC
// An attempt was made to join a Secured Network, but the device did not
// receive a Network Key.
#define EMBER_STATUS_NO_NETWORK_KEY_RECEIVED 0xAD
// After a device joined a Secured Network, a Link Key was requested but no
// response was ever received.
#define EMBER_STATUS_NO_LINK_KEY_RECEIVED 0xAE
// An attempt was made to join a Secured Network without a pre-configured
// key, but the Trust Center sent encrypted data using a pre-configured key.
#define EMBER_STATUS_PRECONFIGURED_KEY_REQUIRED 0xAF
// The node has not joined a network.
#define EMBER_STATUS_NOT_JOINED 0x93
// The chosen security level (the value of SECURITY_LEVEL) is not supported
// by the stack.
#define EMBER_STATUS_INVALID_SECURITY_LEVEL 0x95
// A message cannot be sent because the network is currently overloaded.
#define EMBER_STATUS_NETWORK_BUSY 0xA1
// The application tried to send a message using an endpoint that it has not
// defined.
#define EMBER_STATUS_INVALID_ENDPOINT 0xA3
// The application tried to use a binding that has been remotely modified
// and the change has not yet been reported to the application.
#define EMBER_STATUS_BINDING_HAS_CHANGED 0xA4
// An attempt to generate random bytes failed because of insufficient random
// data from the radio.
#define EMBER_STATUS_INSUFFICIENT_RANDOM_DATA 0xA5
// There was an error in trying to encrypt at the APS Level. This could
// result from either an inability to determine the long address of the
// recipient from the short address (no entry in the binding table) or there
// is no link key entry in the table associated with the destination, or
// there was a failure to load the correct key into the encryption core.
// TRUST_CENTER_MASTER_KEY_NOT_SET 0xA7 There was an attempt to form a
// network using commercial security without setting the Trust Center master
// key first.
#define EMBER_STATUS_APS_ENCRYPTION_ERROR 0xA6
// There was an attempt to form a network using commercial security without setting
// the Trust Center master key first.
#define EMBER_STATUS_TRUST_CENTER_MASTER_KEY_NOT_SET 0xA7
// There was an attempt to form or join a network with security without
// calling emberSetInitialSecurityState() first.
#define EMBER_STATUS_SECURITY_STATE_NOT_SET 0xA8
// There was an attempt to set an entry in the key table using an invalid
// long address. An entry cannot be set using either the local device's or
// Trust Center's IEEE address. Or an entry already exists in the table with
// the same IEEE address. An Address of all zeros or all F's are not valid
// addresses in 802.15.4.
#define EMBER_STATUS_KEY_TABLE_INVALID_ADDRESS 0xB3
// There was an attempt to set a security configuration that is not valid
// given the other security settings.
#define EMBER_STATUS_SECURITY_CONFIGURATION_INVALID 0xB7
// There was an attempt to broadcast a key switch too quickly after
// broadcasting the next network key. The Trust Center must wait at least a
// period equal to the broadcast timeout so that all routers have a chance
// to receive the broadcast of the new network key.
#define EMBER_STATUS_TOO_SOON_FOR_SWITCH_KEY 0xB8
// The message could not be sent because the link key corresponding to the
// destination is not authorized for use in APS data messages. APS Commands
// (sent by the stack) are allowed. To use it for encryption of APS data
// messages it must be authorized using a key agreement protocol (such as
// CBKE).
#define EMBER_STATUS_KEY_NOT_AUTHORIZED 0xBB
// The security data provided was not valid, or an integrity check failed.
#define EMBER_STATUS_SECURITY_DATA_INVALID 0xBD
// A ZigBee route error command frame was received indicating that a source
// routed message from this node failed en route.
#define EMBER_STATUS_SOURCE_ROUTE_FAILURE 0xA9
// A ZigBee route error command frame was received indicating that a message
// sent to this node along a many-to-one route failed en route. The route
// error frame was delivered by an ad-hoc search for a functioning route.
#define EMBER_STATUS_MANY_TO_ONE_ROUTE_FAILURE 0xAA
// A critical and fatal error indicating that the version of the stack
// trying to run does not match with the chip it is running on. The software
// (stack) on the chip must be replaced with software that is compatible
// with the chip.
#define EMBER_STATUS_STACK_AND_HARDWARE_MISMATCH 0xB0
// An index was passed into the function that was larger than the valid
// range.
#define EMBER_STATUS_INDEX_OUT_OF_RANGE 0xB1
// The passed key data is not valid. A key of all zeros or all F's are reserved
// values and cannot be used.
#define EMBER_STATUS_KEY_INVALID 0xB2
// There are no empty entries left in the table.
#define EMBER_STATUS_TABLE_FULL 0xB4
// The requested table entry has been erased and contains no valid data.
#define EMBER_STATUS_TABLE_ENTRY_ERASED 0xB6
// The requested function cannot be executed because the library that
// contains the necessary functionality is not present.
#define EMBER_STATUS_LIBRARY_NOT_PRESENT 0xB5
// The stack accepted the command and is currently processing the request.
// The results will be returned via an appropriate handler.
#define EMBER_STATUS_OPERATION_IN_PROGRESS 0xBA
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_0 0xF0
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_1 0xF1
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_2 0xF2
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_3 0xF3
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_4 0xF4
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_5 0xF5
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_6 0xF6
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_7 0xF7
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_8 0xF8
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_9 0xF9
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_10 0xFA
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_11 0xFB
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_12 0xFC
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_13 0xFD
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_14 0xFE
// This error is reserved for customer application use.  This will never be
// returned from any portion of the network stack or HAL.
#define EMBER_STATUS_APPLICATION_ERROR_15 0xFF


// Describes the presence of valid data within the EmberKeyStruct structure. //

// The key has a sequence number associated with it.
#define EMBER_BITMASK_KEY_HAS_SEQUENCE_NUMBER 0x0001
// The key has an outgoing frame counter associated with it.
#define EMBER_BITMASK_KEY_HAS_OUTGOING_FRAME_COUNTER 0x0002
// The key has an incoming frame counter associated with it.
#define EMBER_BITMASK_KEY_HAS_INCOMING_FRAME_COUNTER 0x0004
// The key has a Partner IEEE address associated with it.
#define EMBER_BITMASK_KEY_HAS_PARTNER_EUI64 0x0008
// This indicates the key is authorized for use in APS data messages.
// If the key is not authorized for use in APS data messages it has not
// yet gone through a key agreement protocol, such as CBKE (i.e. ECC)
#define EMBER_BITMASK_KEY_IS_AUTHORIZED 0x0010
// This indicates that the partner associated with the link is a sleepy
// end device.  This bit is set automatically if the local device
// hears a device announce from the partner indicating it is not
// an 'RX on when idle' device.
#define EMBER_BITMASK_KEY_PARTNER_IS_SLEEPY 0x0020
// This indicates that the transient key which is being added is unconfirmed. This
// bit is set when we add a transient key while the EmberTcLinkKeyRequestPolicy is
// EMBER_ALLOW_TC_LINK_KEY_REQUEST_AND_GENERATE_NEW_KEY
#define EMBER_BITMASK_UNCONFIRMED_TRANSIENT_KEY 0x0040


// The status of the attempt to establish a key. //

#define EMBER_STATUS_KEY_STATUS_NONE 0x00
#define EMBER_STATUS_APP_LINK_KEY_ESTABLISHED 0x01
#define EMBER_STATUS_APP_MASTER_KEY_ESTABLISHED 0x02  // removed in ver. 5
#define EMBER_STATUS_TRUST_CENTER_LINK_KEY_ESTABLISHED 0x03
#define EMBER_STATUS_KEY_ESTABLISHMENT_TIMEOUT 0x04
#define EMBER_STATUS_KEY_TABLE_FULL 0x05
#define EMBER_STATUS_TC_RESPONDED_TO_KEY_REQUEST 0x06
#define EMBER_STATUS_TC_APP_KEY_SENT_TO_REQUESTER 0x07
#define EMBER_STATUS_TC_RESPONSE_TO_KEY_REQUEST_FAILED 0x08
#define EMBER_STATUS_TC_REQUEST_KEY_TYPE_NOT_SUPPORTED 0x09
#define EMBER_STATUS_TC_NO_LINK_KEY_FOR_REQUESTER 0x0A
#define EMBER_STATUS_TC_REQUESTER_EUI64_UNKNOWN 0x0B
#define EMBER_STATUS_TC_RECEIVED_FIRST_APP_KEY_REQUEST 0x0C
#define EMBER_STATUS_TC_TIMEOUT_WAITING_FOR_SECOND_APP_KEY_REQUEST 0x0D
#define EMBER_STATUS_TC_NON_MATCHING_APP_KEY_REQUEST_RECEIVED 0x0E
#define EMBER_STATUS_TC_FAILED_TO_SEND_APP_KEYS 0x0F
#define EMBER_STATUS_TC_FAILED_TO_STORE_APP_KEY_REQUEST 0x10
#define EMBER_STATUS_TC_REJECTED_APP_KEY_REQUEST 0x11
#define EMBER_STATUS_TC_FAILED_TO_GENERATE_NEW_KEY 0x12
#define EMBER_STATUS_TC_FAILED_TO_SEND_TC_KEY 0x13
#define EMBER_STATUS_TRUST_CENTER_IS_PRE_R21 0x1E
#define EMBER_STATUS_TC_REQUESTER_VERIFY_KEY_TIMEOUT 0x32
#define EMBER_STATUS_TC_REQUESTER_VERIFY_KEY_FAILURE 0x33
#define EMBER_STATUS_TC_REQUESTER_VERIFY_KEY_SUCCESS 0x34
#define EMBER_STATUS_VERIFY_LINK_KEY_FAILURE 0x64
#define EMBER_STATUS_VERIFY_LINK_KEY_SUCCESS 0x65


// Incoming message types. //

// Unicast.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_UNICAST 0x00
// Unicast reply.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_UNICAST_REPLY 0x01
// Multicast.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_MULTICAST 0x02
// Multicast sent by the local device.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_MULTICAST_LOOPBACK 0x03
// Broadcast.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_BROADCAST 0x04
// Broadcast sent by the local device.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_BROADCAST_LOOPBACK 0x05
// Many to one route request.
#define EMBER_INCOMING_MSG_TYPE_INCOMING_MANY_TO_ONE_ROUTE_REQUEST 0x06


// Outgoing message types. //

// Unicast sent directly to an EmberNodeId.
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_DIRECT 0x00
// Unicast sent using an entry in the address table.
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_VIA_ADDRESS_TABLE 0x01
// Unicast sent using an entry in the binding table.
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_VIA_BINDING 0x02
// Multicast message. This value is passed to emberMessageSentHandler()
// only. It may not be passed to emberSendUnicast().
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_MULTICAST 0x03
// Aliased multicast message. This value is passed to emberMessageSentHandler() only.
// It may not be passed to emberSendUnicast().
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_MULTICAST_WITH_ALIAS 0x04
// Aliased Broadcast message. This value is passed to emberMessageSentHandler() only.
// It may not be passed to emberSendUnicast().
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_BROADCAST_WITH_ALIAS 0x05
// Broadcast message. This value is passed to emberMessageSentHandler()
// only. It may not be passed to emberSendUnicast().
#define EMBER_OUTGOING_MSG_TYPE_OUTGOING_BROADCAST 0x06


// An energy scan scans each channel for its RSSI value.
#define EZSP_ENERGY_SCAN 0x00
// An active scan scans each channel for available networks.
#define EZSP_ACTIVE_SCAN 0x01


// 0 No options.
#define EMBER_APS_OPTION_NONE 0x000
// Send the message using APS Encryption, using the Link
// Key shared with the destination node to encrypt the data
// at the APS Level.
#define EMBER_APS_OPTION_ENCRYPTION 0x0020
// Resend the message using the APS retry mechanism.
#define EMBER_APS_OPTION_RETRY 0x0040
// Causes a route discovery to be initiated if no route to the
// destination is known.
#define EMBER_APS_OPTION_ENABLE_ROUTE_DISCOVERY 0x0100
// Causes a route discovery to be initiated even if one is
// known.
#define EMBER_APS_OPTION_FORCE_ROUTE_DISCOVERY 0x0200
// Include the source EUI64 in the network frame.
#define EMBER_APS_OPTION_SOURCE_EUI64 0x0400
// Include the destination EUI64 in the network frame.
#define EMBER_APS_OPTION_DESTINATION_EUI64 0x0800
// Send a ZDO request to discover the node ID of the
// destination, if it is not already know.
#define EMBER_APS_OPTION_ENABLE_ADDRESS_DISCOVERY 0x1000
// Reserved.
#define EMBER_APS_OPTION_POLL_RESPONSE 0x2000
// This incoming message is a ZDO request not handled by
// the EmberZNet stack, and the application is responsible
// for sending a ZDO response. This flag is used only when
// the ZDO is configured to have requests handled by the
// application. See the
// EZSP_CONFIG_APPLICATION_ZDO_FLAGS
// configuration parameter for more information.
#define EMBER_APS_OPTION_ZDO_RESPONSE_REQUIRED 0x4000
// This message is part of a fragmented message. This
// option may only be set for unicasts. The groupId field
// gives the index of this fragment in the low-order byte. If
// the low-order byte is zero this is the first fragment and the
// high-order byte contains the number of fragments in the
// message
#define EMBER_APS_OPTION_FRAGMENT 0x8000


// EmberCounterType //

// The number of times we dropped a packet due to
// the packet-validate library checking a packet and
// rejecting it due to length or other formatting
// problems.
#define EMBER_COUNTER_PACKET_VALIDATE_LIBRARY_DROPPED_COUNT 30
// The number of times the NWK retry queue is full
// and a new message failed to be added.
#define EMBER_COUNTER_TYPE_NWK_RETRY_OVERFLOW 31
// The number of times the PHY layer was unable to
// transmit due to a failed CCA.
#define EMBER_COUNTER_PHY_CCA_FAIL_COUNT 32
// The number of times an NWK broadcast was
// dropped because the broadcast table was full.
#define EMBER_COUNTER_BROADCAST_TABLE_FULL 33
// The number of low priority packet traffic arbitration requests.
#define EMBER_COUNTER_PTA_LO_PRI_REQUESTED 34
// The number of high priority packet traffic arbitration requests.
#define EMBER_COUNTER_PTA_HI_PRI_REQUESTED 35
// The number of low priority packet traffic arbitration requests denied.
#define EMBER_COUNTER_PTA_LO_PRI_DENIED 36
// The number of high priority packet traffic arbitration requests denied.
#define EMBER_COUNTER_PTA_HI_PRI_DENIED 37
// The number of aborted low priority packet traffic arbitration transmissions.
#define EMBER_COUNTER_PTA_LO_PRI_TX_ABORTED 38
// The number of aborted high priority packet traffic arbitration transmissions.
#define EMBER_COUNTER_PTA_HI_PRI_TX_ABORTED 39
// A placeholder giving the number of Ember counter types
#define EMBER_COUNTER_TYPE_COUNT 40


// SL Statuses

#define SL_STATUS_OK    0x0000  ///< No error.
#define SL_STATUS_FAIL  0x0001  ///< Generic error.

// State Errors
#define SL_STATUS_INVALID_STATE         0x0002  ///< Generic invalid state error.
#define SL_STATUS_NOT_READY             0x0003  ///< Module is not ready for requested operation.
#define SL_STATUS_BUSY                  0x0004  ///< Module is busy and cannot carry out requested operation.
#define SL_STATUS_IN_PROGRESS           0x0005  ///< Operation is in progress and not yet complete (pass or fail).
#define SL_STATUS_ABORT                 0x0006  ///< Operation aborted.
#define SL_STATUS_TIMEOUT               0x0007  ///< Operation timed out.
#define SL_STATUS_PERMISSION            0x0008  ///< Operation not allowed per permissions.
#define SL_STATUS_WOULD_BLOCK           0x0009  ///< Non-blocking operation would block.
#define SL_STATUS_IDLE                  0x000A  ///< Operation/module is Idle, cannot carry requested operation.
#define SL_STATUS_IS_WAITING            0x000B  ///< Operation cannot be done while construct is waiting.
#define SL_STATUS_NONE_WAITING          0x000C  ///< No task/construct waiting/pending for that action/event.
#define SL_STATUS_SUSPENDED             0x000D  ///< Operation cannot be done while construct is suspended.
#define SL_STATUS_NOT_AVAILABLE         0x000E  ///< Feature not available due to software configuration.
#define SL_STATUS_NOT_SUPPORTED         0x000F  ///< Feature not supported.
#define SL_STATUS_INITIALIZATION        0x0010  ///< Initialization failed.
#define SL_STATUS_NOT_INITIALIZED       0x0011  ///< Module has not been initialized.
#define SL_STATUS_ALREADY_INITIALIZED   0x0012  ///< Module has already been initialized.
#define SL_STATUS_DELETED               0x0013  ///< Object/construct has been deleted.
#define SL_STATUS_ISR                   0x0014  ///< Illegal call from ISR.
#define SL_STATUS_NETWORK_UP            0x0015  ///< Illegal call because network is up.
#define SL_STATUS_NETWORK_DOWN          0x0016  ///< Illegal call because network is down.
#define SL_STATUS_NOT_JOINED            0x0017  ///< Failure due to not being joined in a network.
#define SL_STATUS_NO_BEACONS            0x0018  ///< Invalid operation as there are no beacons.

// Allocation/ownership Errors
#define SL_STATUS_ALLOCATION_FAILED   0x0019  ///< Generic allocation error.
#define SL_STATUS_NO_MORE_RESOURCE    0x001A  ///< No more resource available to perform the operation.
#define SL_STATUS_EMPTY               0x001B  ///< Item/list/queue is empty.
#define SL_STATUS_FULL                0x001C  ///< Item/list/queue is full.
#define SL_STATUS_WOULD_OVERFLOW      0x001D  ///< Item would overflow.
#define SL_STATUS_HAS_OVERFLOWED      0x001E  ///< Item/list/queue has been overflowed.
#define SL_STATUS_OWNERSHIP           0x001F  ///< Generic ownership error.
#define SL_STATUS_IS_OWNER            0x0020  ///< Already/still owning resource.

// Invalid Parameters Errors
#define SL_STATUS_INVALID_PARAMETER       0x0021  ///< Generic invalid argument or consequence of invalid argument.
#define SL_STATUS_NULL_POINTER            0x0022  ///< Invalid null pointer received as argument.
#define SL_STATUS_INVALID_CONFIGURATION   0x0023  ///< Invalid configuration provided.
#define SL_STATUS_INVALID_MODE            0x0024  ///< Invalid mode.
#define SL_STATUS_INVALID_HANDLE          0x0025  ///< Invalid handle.
#define SL_STATUS_INVALID_TYPE            0x0026  ///< Invalid type for operation.
#define SL_STATUS_INVALID_INDEX           0x0027  ///< Invalid index.
#define SL_STATUS_INVALID_RANGE           0x0028  ///< Invalid range.
#define SL_STATUS_INVALID_KEY             0x0029  ///< Invalid key.
#define SL_STATUS_INVALID_CREDENTIALS     0x002A  ///< Invalid credentials.
#define SL_STATUS_INVALID_COUNT           0x002B  ///< Invalid count.
#define SL_STATUS_NOT_FOUND               0x002C  ///< Item could not be found.
#define SL_STATUS_ALREADY_EXISTS          0x002D  ///< Item already exists.

// IO/Communication Errors
#define SL_STATUS_IO                    0x002E  ///< Generic I/O failure.
#define SL_STATUS_IO_TIMEOUT            0x002F  ///< I/O failure due to timeout.
#define SL_STATUS_TRANSMIT              0x0030  ///< Generic transmission error.
#define SL_STATUS_TRANSMIT_UNDERFLOW    0x0031  ///< Transmit underflowed.
#define SL_STATUS_TRANSMIT_INCOMPLETE   0x0032  ///< Transmit is incomplete.
#define SL_STATUS_TRANSMIT_BUSY         0x0033  ///< Transmit is busy.
#define SL_STATUS_RECEIVE               0x0034  ///< Generic reception error.
#define SL_STATUS_OBJECT_READ           0x0035  ///< Failed to read on/via given object.
#define SL_STATUS_OBJECT_WRITE          0x0036  ///< Failed to write on/via given object.
#define SL_STATUS_MESSAGE_TOO_LONG      0x0037  ///< Message is too long.

// EEPROM/Flash Errors
#define SL_STATUS_EEPROM_MFG_VERSION_MISMATCH     0x0038  ///<
#define SL_STATUS_EEPROM_STACK_VERSION_MISMATCH   0x0039  ///<
#define SL_STATUS_FLASH_WRITE_INHIBITED           0x003A  ///< Flash write is inhibited.
#define SL_STATUS_FLASH_VERIFY_FAILED             0x003B  ///< Flash verification failed.
#define SL_STATUS_FLASH_PROGRAM_FAILED            0x003C  ///< Flash programming failed.
#define SL_STATUS_FLASH_ERASE_FAILED              0x003D  ///< Flash erase failed.

// MAC Errors
#define SL_STATUS_MAC_NO_DATA                   0x003E  ///<
#define SL_STATUS_MAC_NO_ACK_RECEIVED           0x003F  ///<
#define SL_STATUS_MAC_INDIRECT_TIMEOUT          0x0040  ///<
#define SL_STATUS_MAC_UNKNOWN_HEADER_TYPE       0x0041  ///<
#define SL_STATUS_MAC_ACK_HEADER_TYPE           0x0042  ///<
#define SL_STATUS_MAC_COMMAND_TRANSMIT_FAILURE  0x0043  ///<

// CLI_STORAGE Errors
#define SL_STATUS_CLI_STORAGE_NVM_OPEN_ERROR    0x0044  ///< Error in open NVM

// Security status codes
#define SL_STATUS_SECURITY_IMAGE_CHECKSUM_ERROR 0x0045  ///< Image checksum is not valid.
#define SL_STATUS_SECURITY_DECRYPT_ERROR        0x0046  ///< Decryption failed

// Command status codes
#define SL_STATUS_COMMAND_IS_INVALID            0x0047  ///< Command was not recognized
#define SL_STATUS_COMMAND_TOO_LONG              0x0048  ///< Command maximum length exceeded
#define SL_STATUS_COMMAND_INCOMPLETE            0x0049  ///< Data received does not form a complete command

// Ember Node Types
// Device is not joined.
#define EMBER_NODE_TYPE_UNKNOWN_DEVICE    0x00 
// Will relay messages and can act as a parent to other nodes.
#define EMBER_NODE_TYPE_COORDINATOR       0x01 
// Will relay messages and can act as a parent to other nodes.
#define EMBER_NODE_TYPE_ROUTER            0x02 
// Communicates only with its parent and will not relay messages.
#define EMBER_NODE_TYPE_END_DEVICE        0x03 
// An end device whose radio can be turned off to save power. The application must poll to receive messages.
#define EMBER_NODE_TYPE_SLEEPY_END_DEVICE 0x04 


// EmberConcentratorType 
// A concentrator with insufficient memory to store source
// routes for the entire network. Route records are sent to the
// concentrator prior to every inbound APS unicast.
#define EMBER_LOW_RAM_CONCENTRATOR  0xFFF8
// A concentrator with sufficient memory to store source
// routes for the entire network. Remote nodes stop sending
// route records once the concentrator has successfully
// received one.
#define EMBER_HIGH_RAM_CONCENTRATOR 0xFFF9

// MulticastTableEntry //
#define MULTICAST_TABLE_ENTRY_ID_OFFSET         0
#define MULTICAST_TABLE_ENTRY_ENDPOINT_OFFSET   2
#define MULTICAST_TABLE_ENTRY_NETWORK_ID_OFFSET 3

#endif