//
//  ZEnoceanLib.h
//  z-enocean
//
//  Created by Alex Skalozub on 11/03/14.
//
//  Copyright (c) 2014 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//

#ifndef zenocean_lib_h
#define zenocean_lib_h

#include "ZMalloc.h"
#include "ZLog.h"
#include "ZDefsPublic.h"

ZWEXPORT ZWError zeno_init(ZEnocean *pzeno, ZWCSTR port, ZWCSTR config_folder, ZWCSTR name, ZWLog logger);

ZWEXPORT void zeno_terminate(ZEnocean *pzeno);

ZWEXPORT ZWError zeno_start(ZEnocean zeno, ZEnoceanTerminationCallback termination_callback, void* arg);

ZWEXPORT ZWError zeno_stop(ZEnocean zeno);

ZWEXPORT ZWLog zeno_get_logger(ZEnocean zeno);

ZWEXPORT ZWCSTR zeno_get_name(ZEnocean zeno);
    
ZWEXPORT ZWBOOL zeno_is_idle(ZEnocean zeno);

ZWEXPORT ZWBOOL zeno_is_running(ZEnocean zeno);

// Returns the time of last change in the list of devices
//
// @param zeno:
// ZEnocean object instance
//
ZWEXPORT time_t zeno_get_devices_update_time(const ZEnocean zeno);

ZWEXPORT ZWError zeno_device_add_callback(const ZEnocean zeno, ZEDeviceChangeType mask, ZEnoceanDeviceCallback callback, void* arg);
    
ZWEXPORT ZWError zeno_discover(ZEnocean zeno);
    
ZWEXPORT ZWError zeno_device_remove_callback(const ZEnocean zeno, ZEnoceanDeviceCallback callback, void* arg);
    
// Search a Data holder by name in controller Data holder
//
// @param: zeno
// Enocean object instance
//
// @param: path
// Path to search for (dot separated)
//
ZWEXPORT ZDataHolder zeno_find_controller_data(const ZEnocean zeno, const char *path);

// Search a Data holder by name in device Data holder
//
// @param: zeno
// Enocean object instance
//
// @param: device_id
// Node Id
//
// @param: path
// Path to search for (dot separated)
//
ZWEXPORT ZDataHolder zeno_find_device_data(const ZEnocean zeno, unsigned int device_id, const char *path);

// Returns 0 terminated list of registered devices Node Id.
// NULL is returned if incorrect ZEnocean object is given.
//
// @param zeno:
// Enocean object instance
//
ZWEXPORT ZEDevicesList zeno_devices_list(const ZEnocean zeno);

// Free list of registered devices Node Id previously allocated by zeno_devices_list.
//
// @param list:
// List to be freed
//
ZWEXPORT void zeno_devices_list_free(ZEDevicesList list);

// Remove device by id
//
// @param: zeno
// Enocean object instance
//
// @param: device_id
// Node Id
//
ZWEXPORT ZWError zeno_device_delete(ZEnocean zeno, unsigned int device_id);

// Add device
//
// @param: zeno
// Enocean object instance
//
// @param: startStop
// TRUE to start adding, FALSE to stop adding
//
// @param: rorg
// EEP RORG of the device to be added
//
// @param: func
// EEP FUNC of the device to be added
//
// @param: type
// EEP TYPE of the device to be added
//
ZWEXPORT ZWError zeno_add_device(ZEnocean zeno, ZWBOOL startStop, ZWBYTE rorg, ZWBYTE func, ZWBYTE type);

// Save devices data
//
// @param: zeno
// Enocean object instance
//
ZWEXPORT ZWError zeddx_save_to_xml(const ZEnocean zeno);

#endif
