//
//  ZSerialIO.h
//  Part of Z-Way.C library
//
//  Created by Alex Skalozub on 1/9/12.
//  Based on Z-Way source code written by Christian Paetz and Poltorak Serguei
//
//  Copyright (c) 2012 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zway_serial_io_h
#define zway_serial_io_h

#include "ZPlatform.h"
#include "ZLog.h"

enum _ZW_HANDLE_TYPE {
    ZW_HANDLE_TYPE_INVALID = 0,
    ZW_HANDLE_TYPE_TTY,
    ZW_HANDLE_TYPE_UNIX,
    ZW_HANDLE_TYPE_BLE
};
typedef enum _ZW_HANDLE_TYPE ZW_HANDLE_TYPE;

// Available UART speeds
#define UART_SPEEDS  {  1200,  2400,  1800,  4800,  9600, 19200,   38400,  57600,  115200,  230400,  460800,  921600 }
#define UART_BSPEEDS { B1200, B2400, B1800, B4800, B9600, B19200, B38400, B57600, B115200, B230400, B460800, B921600 }

struct _ZWHANDLE
{
    ZW_FILE_HANDLE fd;
    ZW_HANDLE_TYPE type;
    void *data; // for protocol custom data - memory should be handled by the protocol manager
};
typedef struct _ZWHANDLE *ZWHANDLE;

ZWEXPORT_PRIVATE ZWBOOL zio_open(ZWLog logger, ZWCSTR name, ZWHANDLE *handle);

ZWEXPORT_PRIVATE void zio_close(ZWLog logger, ZWHANDLE *handle);

ZWEXPORT_PRIVATE ZW_HANDLE_TYPE zio_type(ZWHANDLE handle);

ZWEXPORT_PRIVATE void zio_configure(ZWLog logger, ZWHANDLE handle, speed_t baudRate);

ZWEXPORT_PRIVATE int zio_write(ZWLog logger, ZWHANDLE handle, const ZWBYTE *buffer, size_t size);

ZWEXPORT_PRIVATE int zio_read(ZWLog logger, ZWHANDLE handle, ZWBYTE *buffer, size_t size);

ZWEXPORT_PRIVATE ZWBOOL zio_check_speed(ZWLog log, speed_t speed);

#endif
