//
//  ZDataExp.h
//  Part of Z-Matter.C library
//
//  Created by Kirill Rogozhkin on 3/10/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2022 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zmatter_zdata_exp_h
#define zmatter_zdata_exp_h

#include "ZPlatform.h"
#include "ZMatterLib.h"

#ifdef __cplusplus
extern "C" {
#endif

// Search a Data holder by name in controller Data holder
//
// @param: zmatter
// Z-Matter object endpoint
//
// @param: path
// Path to search for (dot separated)
// NULL or empty string to get root object
//
ZWEXPORT ZDataHolder zmatter_find_controller_data(const ZMatter zmatter, const char *path);

// Search a Data holder by name in device Data holder
//
// @param: zmatter
// Z-Matter object endpoint
//
// @param: device_id
// NWK Id
//
// @param: path
// Path to search for (dot separated)
// NULL or empty string to get root object
//
ZWEXPORT ZDataHolder zmatter_find_device_data(const ZMatter zmatter, ZMNWK device_id, const char *path);

// Search a Data holder by name in endpoint Data holder
//
// @param: zmatter
// Z-Matter object endpoint
//
// @param: device_id
// NWK Id
//
// @param: endpoint_id
// endpoint Id
//
// @param: path
// Path to search for (dot separated)
// NULL or empty string to get root object
//
ZWEXPORT ZDataHolder zmatter_find_device_endpoint_data(const ZMatter zmatter, ZMNWK device_id, ZWBYTE endpoint_id, const char *path);

// Search a Data holder by name in Command Class Data holder
//
// @param: zmatter
// Z-Matter object endpoint
//
// @param: device_id
// NWK Id
//
// @param: endpoint_id
// endpoint Id
//
// @param: cc_id
// Command Class Id
//
// @param: path
// Path to search for (dot separated)
// NULL or empty string to get root object
//
ZWEXPORT ZDataHolder zmatter_find_device_endpoint_cc_data(const ZMatter zmatter, ZMNWK device_id, ZWBYTE endpoint_id, ZMClusterId frame_id, const char *path);

#ifdef __cplusplus
}
#endif

#endif