//
//  ZDefsPublic.h
//  Part of Z-Matter.C library
//
//  Created by Kirill Rogozhkin on 20/9/22.
//  Based on Z-Way source code written by Alex Skalozub and Serguei Poltorak
//
//  Copyright (c) 2012 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zmatter_defs_public_h
#define zmatter_defs_public_h

#include "ZPlatform.h"
#include "ZData.h"
#include "ZMalloc.h"

#ifdef __cplusplus
extern "C" {
#endif

// Z-Matter defs //

// State of controller
enum _ZMControllerState
{
    ZMIdle = 0,             // Normal operation
    
    ZMAddReady = 1,         // Ready to include device
    ZMAddNodeFound = 2,     // Node found
    ZMAddLearning = 3,      // Assigning NodeId
    ZMAddDone = 4,          // Inclusion done
    
    ZMRemoveReady = 5,      // Ready to exclude device
    ZMRemoveNodeFound = 6,  // Node found
    ZMRemoveLearning = 7,   // Assigning NodeId to zero

    ZMResetting = 20,       // Resetting controller
};

// Use this type to save controller state
typedef int ZMState;

// Types of Device change events
enum _ZMDeviceChangeType
{
    ZMDeviceAdded = 0x01,
    ZMDeviceRemoved = 0x02,
    ZMEndPointAdded = 0x04,
    ZMEndPointRemoved = 0x08,
    ZMClusterAdded = 0x10,
    ZMClusterRemoved = 0x20,
    ZMDDXSaved = 0x100, // this callback notifies on ZDDX data change (to allow main program to purge buffers to disk/flash). For this event deviceId = 0
    ZMEnumerateExisting = 0x200, // this flag makes callback immediately fire for all existing devices classes
};

// Device IEEE
typedef uint8_t ZMEUI64[8];

typedef uint8_t ZMKey16[16];

// ZigMatter Network Layer Address
typedef uint16_t ZMNWK;

// ZTimer defs
struct _ZTimer;
typedef struct _ZTimer *ZTimer;

// Endpoint Id
typedef uint8_t ZMEndpointId;

// Cluster Id
typedef uint16_t ZMClusterId;

// Attribute Id
typedef uint16_t ZMAttributeId;

// AttributeSet Id
typedef uint16_t ZMAttributeSetId;

// Command Id 
typedef ZWBYTE ZMCommandId;

// Manufacturer Code type
typedef uint16_t ZMManufacturerCode;

// List of NULL terminated registered devices Node Id
typedef ZMNWK * ZMDevicesList;

// List of NULL terminated registered instances Id for device
typedef ZMEndpointId * ZMEndPointsList;

// List of NULL terminated registered clusters Id for endpoint of device
typedef ZMClusterId * ZMClustersList;

// Z-Matter cluster holder
struct _ZMatter;
typedef struct _ZMatter *ZMatter;

// Definition of termination callback function
typedef void (*ZMTerminationCallback)(const ZMatter zmatter, void* arg);

// Use this type to store Device change event type
typedef int ZMDeviceChangeType;

// Definition of device change callback function
typedef void (*ZMDeviceCallback)(const ZMatter zmatter, ZMDeviceChangeType type, ZMNWK node_id, ZWBYTE endpoint_id, ZMClusterId cluster_id, void *arg);

// Definition of custom callback for EZSP Frame and ZCL calls
typedef void (*ZMJobCustomCallback)(const ZMatter zmatter, ZMClusterId id, void* arg);

#define ZM_GLOBAL_ID_READ_ATTRIBUTES                       0x00
#define ZM_GLOBAL_ID_READ_ATTRIBUTES_RESPONSE              0x01
#define ZM_GLOBAL_ID_WRITE_ATTRIBUTES                      0x02
#define ZM_GLOBAL_ID_WRITE_ATTRIBUTES_UNDIVIDED            0x03
#define ZM_GLOBAL_ID_WRITE_ATTRIBUTES_RESPONSE             0x04
#define ZM_GLOBAL_ID_WRITE_ATTRIBUTES_NO_RESPONSE          0x05
#define ZM_GLOBAL_ID_CONFIGURE_REPORTING                   0x06
#define ZM_GLOBAL_ID_CONFIGURE_REPORTING_RESPONSE          0x07
#define ZM_GLOBAL_ID_READ_REPORTING_CONFIGURATION          0x08
#define ZM_GLOBAL_ID_READ_REPORTING_CONFIGURATION_RESPONSE 0x09
#define ZM_GLOBAL_ID_REPORT_ATTRIBUTES                     0x0a
#define ZM_GLOBAL_ID_DEFAULT_RESPONSE                      0x0b
#define ZM_GLOBAL_ID_DISCOVER_ATTRIBUTES                   0x0c
#define ZM_GLOBAL_ID_DISCOVER_ATTRIBUTES_RESPONSE          0x0d
#define ZM_GLOBAL_READ_ATTRIBUTES_STRUCTURED               0x0e
#define ZM_GLOBAL_WRITE_ATTRIBUTES_STRUCTURED              0x0f
#define ZM_GLOBAL_WRITE_ATTRIBUTES_STRUCTURED_RESPONSE     0x10
#define ZM_GLOBAL_DISCOVER_COMMANDS_RECEIVED               0x11
#define ZM_GLOBAL_DISCOVER_COMMANDS_RECEIVED_RESPONSE      0x12
#define ZM_GLOBAL_DISCOVER_COMMANDS_GENERATED              0x13
#define ZM_GLOBAL_DISCOVER_COMMANDS_GENERATED_RESPONSE     0x14
#define ZM_GLOBAL_DISCOVER_ATTRIBUTES_EXTENDED             0x15
#define ZM_GLOBAL_DISCOVER_ATTRIBUTES_EXTENDED_RESPONSE    0x16

#define ZM_ENDPOINT_BROADCAST_ID 0xFF

typedef enum
{
    ZM_BROADCAST_ADDR_ALL_DEVICES                 = 0xFFFF,
    ZM_BROADCAST_ADDR_RESERVED_FFFE               = 0xFFFE,
    ZM_BROADCAST_ADDR_RX_ON_WHEN_IDLE             = 0xFFFD,
    ZM_BROADCAST_ADDR_ALL_ROUTERS_AND_COORDINATOR = 0xFFFC,
    ZM_BROADCAST_ADDR_LOW_POWER_ROUTER            = 0xFFFB,
    ZM_BROADCAST_ADDR_RESERVED_FFFA               = 0xFFFA,
    ZM_BROADCAST_ADDR_RESERVED_FFF9               = 0xFFF9,
    ZM_BROADCAST_ADDR_RESERVED_FFF8               = 0xFFF8,
} ZMBroadcastAddress;

#define NWK_NO_CHANNELS  0x00000000
#define NWK_ALL_CHANNELS 0x07FFF800
#define NWK_CHANNEL_11   0x00000800
#define NWK_CHANNEL_12   0x00001000
#define NWK_CHANNEL_13   0x00002000
#define NWK_CHANNEL_14   0x00004000
#define NWK_CHANNEL_15   0x00008000
#define NWK_CHANNEL_16   0x00010000
#define NWK_CHANNEL_17   0x00020000
#define NWK_CHANNEL_18   0x00040000
#define NWK_CHANNEL_19   0x00080000
#define NWK_CHANNEL_20   0x00100000
#define NWK_CHANNEL_21   0x00200000
#define NWK_CHANNEL_22   0x00400000
#define NWK_CHANNEL_23   0x00800000
#define NWK_CHANNEL_24   0x01000000
#define NWK_CHANNEL_25   0x02000000
#define NWK_CHANNEL_26   0x04000000

// Attribute Power Source
typedef enum
{
    ZMatter_PS_Unknown = 0x00,
    ZMatter_PS_MainsSinglePhase = 0x01,
    ZMatter_PS_Mains3Phase = 0x02,
    ZMatter_PS_Battery = 0x03,
    ZMatter_PS_DcSource = 0x04,
    ZMatter_PS_EmergencyMainsConstrants = 0x05,
    ZMatter_PS_EmergencyMainsAndTransferSwitch = 0x06,
} ZMPowerSource;

// Attribute Physical Enviroment
typedef enum
{
    ZMatter_PE_UnspecifiedEnvironment = 0x00,
    ZMatter_PE_MirrorZseProfileOrAtrium = 0x01,
    ZMatter_PE_Bar = 0x02,
    ZMatter_PE_Courtyard = 0x03,
    ZMatter_PE_Bathroom = 0x04,
    ZMatter_PE_Bedroom = 0x05,
    ZMatter_PE_BilliardRoom = 0x06,
    ZMatter_PE_UtilityRoom = 0x07,
    ZMatter_PE_Cellar = 0x08,
    ZMatter_PE_StorageCloset = 0x09,
    ZMatter_PE_Theater = 0x0a,
    ZMatter_PE_Office1 = 0x0b,
    ZMatter_PE_Deck = 0x0c,
    ZMatter_PE_Den = 0x0d,
    ZMatter_PE_DiningRoom = 0x0e,
    ZMatter_PE_ElectricalRoom = 0x0f,
    ZMatter_PE_Elevator = 0x10,
    ZMatter_PE_Entry = 0x11,
    ZMatter_PE_FamilyRoom = 0x12,
    ZMatter_PE_MainFloor = 0x13,
    ZMatter_PE_Upstairs = 0x14,
    ZMatter_PE_Downstairs = 0x15,
    ZMatter_PE_BasementOrLowerLevel = 0x16,
    ZMatter_PE_Gallery = 0x17,
    ZMatter_PE_GameRoom = 0x18,
    ZMatter_PE_Garage = 0x19,
    ZMatter_PE_Gym = 0x1a,
    ZMatter_PE_Hallway = 0x1b,
    ZMatter_PE_House = 0x1c,
    ZMatter_PE_Kitchen = 0x1d,
    ZMatter_PE_LaundryRoom = 0x1e,
    ZMatter_PE_Library = 0x1f,
    ZMatter_PE_MasterBedroom = 0x20,
    ZMatter_PE_MudRoomSmallRoomForCoatsAndBoots = 0x21,
    ZMatter_PE_Nursery = 0x22,
    ZMatter_PE_Pantry = 0x23,
    ZMatter_PE_Office2 = 0x24,
    ZMatter_PE_Outside = 0x25,
    ZMatter_PE_Pool = 0x26,
    ZMatter_PE_Porch = 0x27,
    ZMatter_PE_SewingRoom = 0x28,
    ZMatter_PE_SittingRoom = 0x29,
    ZMatter_PE_Stairway = 0x2a,
    ZMatter_PE_Yard = 0x2b,
    ZMatter_PE_Attic = 0x2c,
    ZMatter_PE_HotTub = 0x2d,
    ZMatter_PE_LivingRoom1 = 0x2e,
    ZMatter_PE_Sauna = 0x2f,
    ZMatter_PE_ShopOrWorkshop = 0x30,
    ZMatter_PE_GuestBedroom = 0x31,
    ZMatter_PE_GuestBath = 0x32,
    ZMatter_PE_PowderRoomHalfBath = 0x33,
    ZMatter_PE_BackYard = 0x34,
    ZMatter_PE_FrontYard = 0x35,
    ZMatter_PE_Patio = 0x36,
    ZMatter_PE_Driveway = 0x37,
    ZMatter_PE_SunRoom = 0x38,
    ZMatter_PE_LivingRoom2 = 0x39,
    ZMatter_PE_Spa = 0x3a,
    ZMatter_PE_Whirlpool = 0x3b,
    ZMatter_PE_Shed = 0x3c,
    ZMatter_PE_EquipmentStorage = 0x3d,
    ZMatter_PE_HobbyOrCraftRoom = 0x3e,
    ZMatter_PE_Fountain = 0x3f,
    ZMatter_PE_Pond = 0x40,
    ZMatter_PE_ReceptionRoom = 0x41,
    ZMatter_PE_BreakfastRoom = 0x42,
    ZMatter_PE_Nook = 0x43,
    ZMatter_PE_Garden = 0x44,
    ZMatter_PE_Balcony = 0x45,
    ZMatter_PE_PanicRoom = 0x46,
    ZMatter_PE_Terrace = 0x47,
    ZMatter_PE_Roof = 0x48,
    ZMatter_PE_Toilet = 0x49,
    ZMatter_PE_ToiletMain = 0x4a,
    ZMatter_PE_OutsideToilet = 0x4b,
    ZMatter_PE_ShowerRoom = 0x4c,
    ZMatter_PE_Study = 0x4d,
    ZMatter_PE_FrontGarden = 0x4e,
    ZMatter_PE_BackGarden = 0x4f,
    ZMatter_PE_Kettle = 0x50,
    ZMatter_PE_Television = 0x51,
    ZMatter_PE_Stove = 0x52,
    ZMatter_PE_Microwave = 0x53,
    ZMatter_PE_Toaster = 0x54,
    ZMatter_PE_Vacuum = 0x55,
    ZMatter_PE_Appliance = 0x56,
    ZMatter_PE_FrontDoor = 0x57,
    ZMatter_PE_BackDoor = 0x58,
    ZMatter_PE_FridgeDoor = 0x59,
    ZMatter_PE_MedicationCabinetDoor = 0x60,
    ZMatter_PE_WardrobeDoor = 0x61,
    ZMatter_PE_FrontCupboardDoor = 0x62,
    ZMatter_PE_OtherDoor = 0x63,
    ZMatter_PE_WaitingRoom = 0x64,
    ZMatter_PE_TriageRoom = 0x65,
    ZMatter_PE_DoctorsOffice = 0x66,
    ZMatter_PE_PatientsPrivateRoom = 0x67,
    ZMatter_PE_ConsultationRoom = 0x68,
    ZMatter_PE_NurseStation = 0x69,
    ZMatter_PE_Ward = 0x6a,
    ZMatter_PE_Corridor = 0x6b,
    ZMatter_PE_OperatingTheatre = 0x6c,
    ZMatter_PE_DentalSurgeryRoom = 0x6d,
    ZMatter_PE_MedicalImagingRoom = 0x6e,
    ZMatter_PE_DecontaminationRoom = 0x6f,
    ZMatter_PE_UnknownEnvironment = 0xff,
} ZMPhysicalEnviroment;

// Cluster Identify //
typedef enum
{
    ZMatter_CI_Blink = 0x00,
    ZMatter_CI_Breathe = 0x01,
    ZMatter_CI_Okay = 0x02,
    ZMatter_CI_ChannelChange = 0x0b,
    ZMatter_CI_FinishEffect = 0xfe,
    ZMatter_CI_StopEffect = 0xff,
} ZMIdentifyEffectId;

typedef enum
{
    ZMatter_IE_Default = 0x00,
} ZMIdentifyEffectVar;

// Attribute value type
enum
{
    ATTRIBUTE_VALUE_TYPE_UINT8,
    ATTRIBUTE_VALUE_TYPE_UINT16,
    ATTRIBUTE_VALUE_TYPE_UINT32,
    ATTRIBUTE_VALUE_TYPE_BOOL
};
#ifdef __cplusplus
}
#endif

#endif