//
//  ZWaySrvLib.h
//  Part of Z-Way.C library
//
//  Created by Serguei Poltorak on 08.10.2024.
//
//  Copyright (c) 2024 Z-Wave.Me
//  All rights reserved
//  info@z-wave.me
//
//  This source file is subject to the terms and conditions of the
//  Z-Wave.Me Software License Agreement which restricts the manner
//  in which it may be used.
//

#ifndef zwaysrv_h
#define zwaysrv_h

#ifdef __cplusplus
extern "C" {
#endif

struct _ZWaySrv;
typedef struct _ZWaySrv *ZWaySrv;

// Allocate and initialize a ZWaySrv object
//
// @param pzwaysrv:
// pointer that gets a ZWaySrv object
// Note! ZWaySrv structure must be filled with zeros first!
//
// @param zway:    
// Z-Way object to work on
// Z-Way should be already started and running (init, start, discovery)
//
// @param srv_port:
// HTTP/WebSocket port to serve on
// If 0, no server will be running
// 
// @param ws_server:
// WebSocket server name to connect to
// Ignored if ws_port is 0
// 
// @param ws_port,:
// port of the WebSocket server to connect to
// If 0, no connection will be established
// 
// @param ws_path:
// WebSocket server path to connect to
// Ignored if ws_port is 0
//
// @param ws_proto
// protocol of the WebSocket to use
// If 0, no connection will be established
//
ZWEXPORT ZWError zwaysrv_init(ZWaySrv * pzwaysrv, ZWay zway, int srv_port, ZWCSTR ws_server, int ws_port, ZWCSTR ws_path, ZWCSTR ws_proto);

// Close all handles and deallocate a ZWaySrv object
//
// @param pzwaysrv:
// pointer to ZWaySrv object
// set to NULL on return
//
ZWEXPORT void zwaysrv_terminate(ZWaySrv *pzwaysrv);

// Start worker thread and start serving
//
// @param zwaysrv:
// ZWaySrv object instance
//
ZWEXPORT ZWError zwaysrv_start(ZWaySrv zwaysrv);

// Stop serving
//
// @param zwaysrv:
// ZWaySrv object instance
//
ZWEXPORT ZWError zwaysrv_stop(ZWaySrv zwaysrv);

#ifdef __cplusplus
}
#endif

#endif
