//
//  Callbacks.h
//  z-way-js
//
//  Created by Alex Skalozub on 7/26/12.
//
//  Copyright (c) 2012 Z-Wave.Me
//

#ifndef zway_js_callbacks_h
#define zway_js_callbacks_h

#include "Dependencies.h"
#include "ZPointer.h"
#include <vector>
#include <map>
#include <memory>

namespace zwjs
{
    class Environment;

    class ZWJSEXPORT SafeValue : public ZRefCountedObject
    {
    public:
        SafeValue(v8::Isolate *isolate, v8::Handle<v8::Value> value) : m_isolate(isolate), m_value(isolate, value)
        {
        }
        
        virtual ~SafeValue()
        {
            m_value.Reset();
        }
        
        v8::Local<v8::Value> GetValue() { return v8::Local<v8::Value>::New(m_isolate, m_value); }
        
        static SafeValue* New(v8::Isolate *isolate, v8::Handle<v8::Value> value)
        {
            if (value.IsEmpty() || value->IsUndefined() || value->IsNull()) return NULL;
            return new SafeValue(isolate, value);
        }
        
    private:
        v8::Isolate *m_isolate;
#ifdef _WINDOWS
        v8::Persistent<v8::Value, v8::CopyablePersistentTraits<v8::Value> > m_value;
#else
        v8::Persistent<v8::Value> m_value;
#endif
    };

    typedef ZRefCountedPointer<SafeValue> SafeValuePtr;

    class ZWJSEXPORT CallbackBase : public ZRefCountedObject
    {
    protected:
        Environment *m_env;
        SafeValuePtr m_callback;
        
        CallbackBase(Environment *env, SafeValuePtr callback) : m_env(env), m_callback(callback) { }
    public:
        virtual void Call() = 0;
    };

    typedef ZRefCountedPointer<CallbackBase> CallbackPtr;

    typedef std::vector< CallbackPtr > CallbackList;

}
    
#endif
