//
//  Environment.h
//  z-way-js
//
//  Created by Alex Skalozub on 20/06/14.
//
//  Copyright (c) 2014 Z-Wave.Me
//

#ifndef zway_js_environment_h
#define zway_js_environment_h

#include "Dependencies.h"
#include "Utility.h"
#include "Thread.h"
#include "RunResult.h"
#include "Callbacks.h"
#include <ZLog.h>
#include "EnvironmentVariable.h"
#include "ZWayException.h"

namespace zwjs
{
    
    class ZWJSEXPORT Environment
    {
    public:
        virtual ~Environment() { }
        
        virtual RunResult Run(const char *script, const char *source = NULL) = 0;
        virtual RunResult RunFile(const char *file) = 0;
        
        virtual void StartThread() = 0;
        virtual void StopThread() = 0;
        
        virtual bool IsThreaded() const = 0;
        virtual bool IsTerminating() const = 0;
        
        virtual void PushCallback(CallbackPtr callback) = 0;
        virtual bool ProcessPendingCallbacks() = 0;
        
        virtual std::string StorageDir() const = 0;
        
        virtual ZWLog GetLogger() = 0;
        virtual void SetLogger(ZWLog logger) = 0;
        
        virtual void Log(ZWLogLevel level, const std::string &message, ZWCSTR category = NULL)
        {
            if (category == NULL)
                category = ZSTR("core");
            
            zlog_write(GetLogger(), category, level, ZSTR("%s"), message.c_str());
        }
        
        virtual v8::Isolate* GetIsolate() const = 0;
        virtual v8::Local<v8::Context> GetContext() const = 0;
        virtual v8::Local<v8::Object> GetGlobal() const = 0;
        
        virtual EnvironmentVariablePtr GetVariable(std::string name) = 0;
        virtual void SetVariable(std::string name, EnvironmentVariablePtr var) = 0;
        
        virtual ThreadPtr CreateThread(std::string name, ThreadFunc func, void* arg) = 0;
        virtual ThreadPtr GetThread(std::string name) = 0;
    };
    
}

#endif
