//
//  MutexLock.h
//  z-way-js
//
//  Created by Alex Skalozub on 23/06/14.
//
//  Copyright (c) 2014 Z-Wave.Me
//

#ifndef zway_js_mutex_lock_h
#define zway_js_mutex_lock_h

#include "Dependencies.h"

namespace zwjs
{
    
    class ZWJSEXPORT MutexLock
    {
    public:
        MutexLock(bool reentrant = false)
        {
#ifndef _WINDOWS
            pthread_mutexattr_init(&m_attr);
            if (reentrant) {
                pthread_mutexattr_settype(&m_attr, PTHREAD_MUTEX_RECURSIVE);
            }
#endif
            LOCK_INIT_EX(&m_lock, &m_attr);
        }
        
        virtual ~MutexLock()
        {
            LOCK_FREE(&m_lock);
#ifndef _WINDOWS
            pthread_mutexattr_destroy(&m_attr);
#endif
        }
        
        void Enter() { LOCK_ENTER(&m_lock); }
        void Leave() { LOCK_EXIT(&m_lock); }
    private:
        ZWMUTEXATTR m_attr;
        ZWMUTEX m_lock;
    };
    
    class ReentrantMutexLock : public MutexLock
    {
    public:
        ReentrantMutexLock() : MutexLock(true) { }
    };
    
}

#endif
