//
//  Thread.h
//  z-way-js
//
//  Created by Alex Skalozub on 21/06/14.
//
//  Copyright (c) 2014 Z-Wave.Me
//

#ifndef zway_js_thread_h
#define zway_js_thread_h

#include "Dependencies.h"
#include "Threading.h"
#include "ZPointer.h"

namespace zwjs
{
    
    class Thread; // forward declaration
    typedef ZRefCountedPointer<Thread> ThreadPtr;
    typedef int (*ThreadFunc)(ThreadPtr info);
    
    class Environment;
    
    class ZWJSEXPORT Thread : public ZRefCountedObject
    {
    public:
        void SetTerminating() { m_terminating = true; }
        void Join();
        void Detach();
        
        bool IsTerminating() const { return m_terminating; }
        void* GetArg() const { return m_arg; }
        void SetArg(void* arg) { m_arg = arg; }
        Environment* GetEnvironment() const { return m_env; }
        std::string GetName() const { return m_name; }
    protected:
        friend class EnvironmentImpl;
        
        Thread(Environment* env, std::string name, ThreadFunc func, void* arg = NULL);
        virtual ~Thread();
        
    private:
        Environment* m_env;
        std::string m_name;
        ThreadFunc m_func;
        void* m_arg;
        
        ZTHREAD m_handle;
        volatile bool m_detached;
        volatile bool m_terminating;

#ifdef _WINDOWS
		HANDLE m_started;
#else
        pthread_mutex_t m_startedSync;
        pthread_cond_t m_started;
        bool m_signalled;
#endif

        static THREAD_RESULT ThreadFuncNative(void* arg);
    };
    
}

#endif
