//
//  Utility.h
//  z-way-js
//
//  Created by Alex Skalozub on 4/29/12.
//
//  Copyright (c) 2012 Z-Wave.Me
//

#ifndef zway_js_utility_h
#define zway_js_utility_h

#include "Dependencies.h"

namespace zwjs
{

    typedef unsigned int CALLBACK_ID;

    std::string ZWJSEXPORT GetString(v8::Handle<v8::Value> value);
    bool ZWJSEXPORT GetBoolean(v8::Handle<v8::Value> value, bool defaultValue = false);

    // helpers to throw string exceptions
    v8::Local<v8::Value> ZWJSEXPORT ThrowException(v8::Isolate *isolate, const char *error);
    v8::Local<v8::Value> ZWJSEXPORT ThrowException(v8::Isolate *isolate, const std::string error);

    // JSON helpers
    v8::Local<v8::Value> ZWJSEXPORT JSON_stringify(v8::Isolate *isolate, v8::Local<v8::Value> value);
    v8::Local<v8::Value> ZWJSEXPORT JSON_parse(v8::Isolate *isolate, v8::Local<v8::String> json_string);

    const char ZWJSEXPORT *GetZWayError(ZWError err);

    v8::Handle<v8::Value> ZWJSEXPORT GetRealValue(v8::Isolate *isolate, v8::Handle<v8::Object> obj, v8::Handle<v8::String> prop);

    std::string ReadFile(const char* path);
    
    // quick string definition macro
#ifdef OLD_V8
    #define V8STR(i, s) v8::String::New(i, s)
    #define V8STRL(i, s, l) v8::String::New(i, (const char*)(s), (int)(l))
    #define V8STRB(i, s, l) v8::String::New(i, (const char*)(s), (int)(l))
#else
    #define V8STR(i, s) v8::String::NewFromUtf8(i, s)
    #define V8STRL(i, s, l) v8::String::NewFromUtf8(i, (const char*)(s), v8::String::kNormalString, (int)(l))
    #define V8STRB(i, s, l) v8::String::NewFromOneByte(i, (const uint8_t*)(s), v8::String::kNormalString, (int)(l))
#endif

#if false // in case old v8 do not support those calls
    #define GET_INTERNAL_PTR(o, n) (v8::Handle<v8::External>::Cast((o)->GetInternalField(n))->Value())
    #define SET_INTERNAL_PTR(o, i, n, v) (o)->SetInternalField(n, v8::External::New(i, v))
#else
    #define GET_INTERNAL_PTR(o, n) ((o)->GetAlignedPointerFromInternalField(n))
    #define SET_INTERNAL_PTR(o, i, n, v) (o)->SetAlignedPointerInInternalField(n, v)
#endif
    
    #define GET_INTERNAL_INT(o, n) (v8::Handle<v8::Integer>::Cast((o)->GetInternalField(n))->Value())
    #define SET_INTERNAL_INT(o, i, n, v) (o)->SetInternalField(n, v8::Int32::New(i, v))
    #define GET_INTERNAL_STR(o, n) (v8::Handle<v8::String>::Cast((o)->GetInternalField(n)))
    #define SET_INTERNAL_STR(o, i, n, v) (o)->SetInternalField(n, V8STR(i, v))

#ifdef OLD_V8

    #define EscapableHandleScope HandleScope
    #define Escape(obj) Close(obj)

    // isolate data ptr access
    #define SET_ISOLATE_DATA(isolate, data) (isolate)->SetData(data)
    #define GET_ISOLATE_DATA(isolate) (isolate)->GetData()

    // function handler
    #define METHOD_DECL(cls, name) static v8::Handle<v8::Value> name(const v8::Arguments &info)
    #define METHOD_IMPL(cls, name) v8::Handle<v8::Value> cls::name(const v8::Arguments &info)

    #define SET_HANDLER(template, isolate, name, handler, flags) template->Set(V8STR(isolate, name), v8::FunctionTemplate::New(isolate, handler), (v8::PropertyAttribute)(flags))

    // property handlers
    #define PROP_GET_DECL(cls, name) static v8::Handle<v8::Value> name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define PROP_GET_IMPL(cls, name) v8::Handle<v8::Value> cls::name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define PROP_SET_DECL(cls, name) static void name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo &info)
    #define PROP_SET_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo &info)

    #define SET_ACCESSOR(template, isolate, name, getter) template->SetAccessor(V8STR(isolate, name), getter)
    #define SET_ACCESSOR2(template, isolate, name, getter, setter) template->SetAccessor(V8STR(isolate, name), getter, setter)
    #define SET_ACCESSOR_HIDDEN(template, isolate, name, getter) template->SetAccessor(V8STR(isolate, name), getter, NULL, v8::Handle<v8::Value>(), v8::DEFAULT, (v8::PropertyAttribute)(v8::DontEnum | v8::ReadOnly))

    // named property handlers
    #define NAMED_GET_DECL(cls, name) static v8::Handle<v8::Value> name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define NAMED_GET_IMPL(cls, name) v8::Handle<v8::Value> cls::name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define NAMED_SET_DECL(cls, name) static v8::Handle<v8::Value> name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo &info)
    #define NAMED_SET_IMPL(cls, name) v8::Handle<v8::Value> cls::name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::AccessorInfo &info)
    #define NAMED_QUERY_DECL(cls, name) static v8::Handle<v8::Integer> name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define NAMED_QUERY_IMPL(cls, name) v8::Handle<v8::Integer> cls::name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define NAMED_DEL_DECL(cls, name) static v8::Handle<v8::Boolean> name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define NAMED_DEL_IMPL(cls, name) v8::Handle<v8::Boolean> cls::name(v8::Local<v8::String> property, const v8::AccessorInfo &info)
    #define NAMED_ENUM_DECL(cls, name) static v8::Handle<v8::Array> name(const v8::AccessorInfo &info)
    #define NAMED_ENUM_IMPL(cls, name) v8::Handle<v8::Array> cls::name(const v8::AccessorInfo &info)

    // indexed property handlers
    #define INDEX_GET_DECL(cls, name) static v8::Handle<v8::Value> name(uint32_t index, const v8::AccessorInfo &info)
    #define INDEX_GET_IMPL(cls, name) v8::Handle<v8::Value> cls::name(uint32_t index, const v8::AccessorInfo &info)

    // helper functions to return different kind of values

    template<class S>
    inline v8::Handle<S> OldReturnValue(const v8::Persistent<S>& handle)
    {
        return v8::Local<S>::New(v8::Isolate::GetCurrent(), handle);
    }

    template<class S>
    inline v8::Handle<S> OldReturnValue(const v8::Handle<S> handle)
    {
        return handle;
    }

    // simple overloads
    v8::Handle<v8::Boolean> OldReturnValue(bool value);
    v8::Handle<v8::Number> OldReturnValue(double value);
    v8::Handle<v8::Integer> OldReturnValue(int32_t value);
    v8::Handle<v8::Integer> OldReturnValue(uint32_t value);
    v8::Handle<v8::Integer> OldReturnValue(v8::PropertyAttribute value);

    #define RETURN(result) { return OldReturnValue(result); }
    #define RETURN_NULL() { return v8::Null(v8::Isolate::GetCurrent()); }
    #define RETURN_UNDEFINED() { return v8::Undefined(v8::Isolate::GetCurrent()); }
    #define RETURN_EMPTY() { return v8::Handle<v8::Value>(); }
    #define RETURN_EMPTY_QUERY() { return v8::Handle<v8::Integer>(); }
    #define RETURN_EMPTY_DEL() { return v8::Handle<v8::Boolean>(); }
    #define RETURN_EMPTY_ENUM() { return v8::Handle<v8::Array>(); }
    #define RETURN2(result) { return; }
    #define RETURN2_UNDEFINED() { return; }

#else // !OLD_V8

    // isolate data ptr access
    #define SET_ISOLATE_DATA(isolate, data) (isolate)->SetData(0, data)
    #define GET_ISOLATE_DATA(isolate) (isolate)->GetData(0)

    // function handler
    #define METHOD_DECL(cls, name) static void name(const v8::FunctionCallbackInfo<v8::Value> &info)
    #define METHOD_IMPL(cls, name) void cls::name(const v8::FunctionCallbackInfo<v8::Value> &info)

    #define SET_HANDLER(template, isolate, name, handler, flags) template->Set(V8STR(isolate, name), v8::FunctionTemplate::New(isolate, handler), (v8::PropertyAttribute)(flags))

    // property handlers
    #define PROP_GET_DECL(cls, name) static void name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define PROP_GET_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define PROP_SET_DECL(cls, name) static void name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::PropertyCallbackInfo<void> &info)
    #define PROP_SET_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::PropertyCallbackInfo<void> &info)

    #define SET_ACCESSOR(template, isolate, name, getter) template->SetAccessor(V8STR(isolate, name), getter)
    #define SET_ACCESSOR2(template, isolate, name, getter, setter) template->SetAccessor(V8STR(isolate, name), getter, setter)
    #define SET_ACCESSOR_HIDDEN(template, isolate, name, getter) template->SetAccessor(V8STR(isolate, name), getter, NULL, v8::Handle<v8::Value>(), v8::DEFAULT, (v8::PropertyAttribute)(v8::DontEnum | v8::ReadOnly))

    // named property handlers
    #define NAMED_GET_DECL(cls, name) static void name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define NAMED_GET_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define NAMED_SET_DECL(cls, name) static void name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define NAMED_SET_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, v8::Local<v8::Value> value, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define NAMED_QUERY_DECL(cls, name) static void name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Integer> &info)
    #define NAMED_QUERY_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Integer> &info)
    #define NAMED_DEL_DECL(cls, name) static void name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Boolean> &info)
    #define NAMED_DEL_IMPL(cls, name) void cls::name(v8::Local<v8::String> property, const v8::PropertyCallbackInfo<v8::Boolean> &info)
    #define NAMED_ENUM_DECL(cls, name) static void name(const v8::PropertyCallbackInfo<v8::Array> &info)
    #define NAMED_ENUM_IMPL(cls, name) void cls::name(const v8::PropertyCallbackInfo<v8::Array> &info)

    // indexed property handlers
    #define INDEX_GET_DECL(cls, name) static void name(uint32_t index, const v8::PropertyCallbackInfo<v8::Value> &info)
    #define INDEX_GET_IMPL(cls, name) void cls::name(uint32_t index, const v8::PropertyCallbackInfo<v8::Value> &info)

    #define RETURN(result) { info.GetReturnValue().Set(result); return; }
    #define RETURN_NULL() { info.GetReturnValue().SetNull(); return; }
    #define RETURN_UNDEFINED() { info.GetReturnValue().SetUndefined(); return; }
    #define RETURN_EMPTY() { return; }
    #define RETURN_EMPTY_QUERY() { return; }
    #define RETURN_EMPTY_DEL() { return; }
    #define RETURN_EMPTY_ENUM() { return; }
    #define RETURN2(result) RETURN(result)
    #define RETURN2_UNDEFINED() RETURN_UNDEFINED()

#endif // OLD_V8

    #define GET_ENVIRONMENT() (Environment*)GET_ISOLATE_DATA(info.GetIsolate())

}

#endif
